<?php
/*
Plugin Name: Desk-Net
Plugin URI: http://support.desk-net.com/hc/en-us/articles/115003386247
Description: Desk-Net integration with WordPress
Version: 1.2.8
Author: Desk-Net GmbH
Author URI: http://www.desk-net.com
*/

require_once plugin_dir_path( __FILE__ ) . 'include/class-desk-net-core.php';
require_once plugin_dir_path( __FILE__ )
             . 'include/class-desk-net-delete-methods.php';
require_once plugin_dir_path( __FILE__ )
             . 'include/class-desk-net-scan-media-files.php';
require_once plugin_dir_path( __FILE__ )
             . 'include/class-desk-net-request-methods.php';

class WP_Desk_Net extends Desk_Net_Core {

	public $desk_net_permalinks = 1;

	/**
	 * @var string post content.
	 */
	protected $post_content = '';

	public function __construct() {
		// Edit links on plugin description page.
		add_filter( 'plugin_row_meta', array(
			$this,
			'desk_net_row_meta',
		), 10, 2 );

		$this->desk_net_permalinks
			= get_option( 'wp_desk_net_id_in_permalink' );

		add_action( 'init', array( $this, 'desk_net_init' ) );
		add_action( 'rest_api_init', array( $this, 'api_hooks' ) );
		add_action( 'admin_enqueue_scripts', array(
			$this,
			'desc_net_css_js',
		) );
		add_action( 'enqueue_block_editor_assets',
			array( $this, 'desk_net_enqueue' ) );

		if ( ! is_admin() && 0 !== (int) $this->desk_net_permalinks ) {
			add_action( 'wp', array( $this, 'desk_net_redirect' ) );
			add_filter( 'post_link', array(
				$this,
				'desk_net_post_link',
			), 10, 3 );
		}

		parent::__construct();
	}

	/**
	 * Perform added the property 'target="_blank"' for author link
	 * and added link 'Visit plugin site'.
	 *
	 * @param array  $links
	 *     The list links.
	 * @param string $file
	 *     The plugin name.
	 *
	 * @return array $query
	 */
	public function desk_net_row_meta( $links, $file ) {
		if ( plugin_basename( __FILE__ ) === $file ) {
			preg_match( '/<a\s/', $links[1], $matches, PREG_OFFSET_CAPTURE );
			$insert_position = $matches[0][1] + 3; // Start index plus 3 chars.

			$links[1] = substr_replace( $links[1], 'target="_blank"',
				$insert_position, 0 );
			$row_meta = '<a href="'
			            . esc_url( 'http://support.desk-net.com/hc/en-us/articles/115003386247' )
			            . '" target="_blank" aria-label="'
			            . esc_attr__( 'Visit plugin site', 'domain' ) . '">'
			            . esc_html__( 'Visit plugin site' ) . '</a>';
			$links[3] = $links[2];
			$links[2] = $row_meta;
		}

		return (array) $links;
	}

	/**
	 * Perform initialize default settings in plugin.
	 */
	public function desk_net_init() {
		if ( 0 !== (int) $this->desk_net_permalinks ) {

			add_rewrite_rule( '([0-9]+)/(.?.+?)?/?$',
				'index.php?page=&post_type=post&name=$matches[2]&desk_net_id=$matches[1]',
				'top' );

			add_filter( 'query_vars', function ( $vars ) {
				$vars[] = 'desk_net_id';

				return $vars;
			} );
		}
		// Post meta.
		$desk_net_post_meta = [
			'story_id',
			'desk_net_description',
			'desk_net_status_name',
			'desk_net_notice_message_id',
			'wp_desk_net_remove_status',
		];

		foreach ( $desk_net_post_meta as $meta_key ) {
			$this->register_post_meta( $meta_key );
		}
	}

	/**
	 * Creating post meta in post content-type.
	 *
	 * @param string $meta_key
	 *     The post meta key.
	 */
	private function register_post_meta( $meta_key ) {
		register_post_meta(
			'post',
			$meta_key,
			array(
				'type'          => 'string',
				'single'        => true,
				'auth_callback' => function () {
					return current_user_can( 'edit_posts' );
				},
				'show_in_rest'  => [
					'prepare_callback' => function ( $value ) {
						return wp_json_encode( $value );
					}
				],
			)
		);
	}

	/**
	 * Perform redirect page in WordPress.
	 */
	public function desk_net_redirect() {
		if ( ! is_single() ) {
			return;
		}

		global $post;

		$url = $this->desk_net_post_link( $post->guid, $post );

		if ( $url !== $post->guid ) {
			wp_safe_redirect( $url, 301 );
			exit;
		}
	}

	/**
	 * Perform API hooks.
	 */
	public function api_hooks() {
		$namespace = 'wp-desk-net/v1';

		register_rest_route( $namespace, '/oauth2/token', array(
			'methods'             => 'POST',
			'callback'            => array( $this, 'auth' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/oauth/token', array(
			'methods'             => 'POST',
			'callback'            => array( $this, 'auth' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/publication', array(
			'methods'             => 'POST',
			'callback'            => array( $this, 'create_publication' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/publication/(?P<post_id>\d+)', array(
			'methods'             => 'PUT',
			'callback'            => array( $this, 'update_publication' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/statuses', array(
			'methods'             => 'POST',
			'callback'            => array( $this, 'statuses' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/files', array(
			'methods'             => 'POST',
			'callback'            => array( $this, 'files' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/files/(?P<cms_id>\d+)', array(
			'methods'             => 'PUT',
			'callback'            => array( $this, 'files' ),
			'permission_callback' => '__return_true'
		) );

		register_rest_route( $namespace, '/publication/(?P<post_id>\d+)', array(
			'methods'             => 'DELETE',
			'callback'            => array( $this, 'delete_publication' ),
			'permission_callback' => '__return_true'
		) );
	}

	/**
	 * Creating post in WordPress by API.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return object
	 */
	public function create_publication( WP_REST_Request $data ) {
		// Validating token.
		if ( ! $this->check_token( $data ) ) {
			return new WP_REST_Response( 'Wrong token', 401 );
		}

		$json_request = $data->get_json_params();

		if ( empty( $json_request ) ) {
			return $this->wp_rest_empty_json_data();
		}

		$json_request['post_id'] = $data['post_id'];

		return $this->create_post( $json_request );
	}

	/**
	 * Updating post in WordPress by API.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return object
	 */
	public function update_publication( WP_REST_Request $data ) {
		// Validating token.
		if ( ! $this->check_token( $data ) ) {
			return new WP_REST_Response( 'Wrong token', 401 );
		}
		// Validating post_id that was got by WP REST API.
		$wp_rest_validation_message
			= $this->validate_wp_rest_by_post_id( $data['post_id'] );

		if ( null !== $wp_rest_validation_message ) {
			return $wp_rest_validation_message;
		}

		// Getting JSON data from request.
		$json_request = $data->get_json_params();

		if ( empty( $json_request ) ) {
			return $this->wp_rest_empty_json_data();
		}

		$json_request['post_id'] = $data['post_id'];
		// Skipping update slug if node was published.
		if ( ( 'publish' === get_post_status( $data['post_id'] )
		       || 'future' === get_post_status( $data['post_id'] ) )
		     && isset( $json_request['slug'] )
		) {
			unset( $json_request['slug'] );
		}

		return $this->update_post( $json_request );
	}

	/**
	 * Deleting post in WordPress by API.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return object $response
	 */
	public function delete_publication( WP_REST_Request $data ) {
		// Validating token.
		if ( ! $this->check_token( $data ) ) {
			return new WP_REST_Response( 'Wrong token', 401 );
		}

		$wp_rest_validation_message
			= $this->validate_wp_rest_by_post_id( $data['post_id'] );

		if ( null !== $wp_rest_validation_message ) {
			return $wp_rest_validation_message;
		}
		// Update Desk-Net Deleted/Removed status.
		update_post_meta( $data['post_id'], 'wp_desk_net_remove_status',
			'removed' );
		// Status Matching.
		$matching_status_remove
			= get_option( 'wp_desk_net_status_desk_net_to_wp_removed' );

		if ( isset( $matching_status_remove ) ) {
			$update_post_fields = array(
				'ID'          => $data['post_id'],
				'post_status' => $matching_status_remove,
			);

			wp_update_post( $update_post_fields );
		}

		return new WP_REST_Response( '', 200 );
	}

	/**
	 * Creating REST response with error message.
	 *
	 * @return object
	 */
	public function wp_rest_empty_json_data() {
		return new WP_REST_Response( 'Error: Empty JSON request data', 400 );
	}

	/**
	 * Validating post ID from a JSON object that was got by WP REST API.
	 *
	 * @param integer $post_id
	 *     The post ID for checking.
	 *
	 * @return object
	 */
	private function validate_wp_rest_by_post_id( $post_id ) {
		$validation_status = 'valid';
		$title             = '';
		$code_status       = 200;

		if ( empty( $post_id ) || ! is_numeric( $post_id ) ) {
			$code_status       = 406;
			$validation_status = 'invalid';
		} else if ( get_post( $post_id ) === null ) {
			$code_status       = 404;
			$validation_status = 'invalid';
		} else if ( get_post_meta( $post_id, 'wp_desk_net_remove_status', true )
		            === 'removed'
		) {
			$title
				               = 'Post was removed by Desk-Net system in previous user actions on Desk-Net side';
			$validation_status = 'invalid';
		}

		return ( $validation_status !== 'valid' )
			? new WP_REST_Response( $title, $code_status ) : null;
	}

	/**
	 * Generating response data in WordPress for REST API.
	 *
	 * @param integer $post_id
	 *     The post ID that was modified.
	 *
	 * @return object
	 */
	private function get_wp_response_data( $post_id ) {
		$post           = get_post( $post_id );
		$post_edit_link = get_edit_post_link( $post_id );

		if ( empty( $post_edit_link ) ) {
			$post_edit_link = admin_url()
			                  . "post.php?post=$post_id&action=edit";
		}

		$response_data = array(
			'id'          => $post_id,
			'cmsEditLink' => $post_edit_link,
			'cmsOpenLink' => $post->guid,
		);

		return new WP_REST_Response( $response_data, 201 );
	}

	/**
	 * Perform updating Desk-Net status list.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return object $response
	 */
	public function statuses( WP_REST_Request $data ) {
		if ( ! $this->check_token( $data ) ) {
			return new WP_REST_Response( 'Wrong token', 401 );
		}

		$json_request = $data->get_json_params();
		$save_statuses_list
		              = get_option( 'wp_desk_net_desk-net-list-active-status' );

		// Update Platform ID.
		if ( isset( $json_request['platform'] ) ) {
			update_option( 'wp_desk_net_platform_id',
				$json_request['platform'] );
		}

		$json_request = $this->check_triggers_export_status( $json_request );

		if ( ! empty( $save_statuses_list ) ) {
			$wp_status_list  = get_post_statuses();
			$delete_elements = new Desk_Net_Delete_Methods();
			$delete_elements->desk_net_shape_deleted_items( $json_request['activeStatuses'],
				$save_statuses_list, $wp_status_list, 'status' );
		}

		if ( ! empty( $json_request['activeStatuses'] ) ) {
			update_option( 'wp_desk_net_desk-net-list-active-status',
				$json_request['activeStatuses'] );
			update_option( 'wp_desk_net_status_wp_to_desk_net_publish', '5' );
			update_option( 'wp_desk_net_status_desk_net_to_wp_5', 'publish' );
		}
		if ( ! empty( $json_request['deactivatedStatuses'] ) ) {
			update_option( 'wp_desk_net_desk-net-list-deactivated-status',
				$json_request['deactivatedStatuses'] );
		}
	}

	/**
	 * Perform Download Attachments.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return array|object
	 */
	public function files( WP_REST_Request $data ) {
		if ( ! $this->check_token( $data ) ) {
			$response_data = array(
				'message' => __( 'Wrong token', 'desk_net' ),
			);

			return new WP_REST_Response( $response_data, 401 );
		}

		$binary_file_string = $data->get_param( 'file' );

		$basic_filename    = $data->get_param( 'filename' );
		$sanitize_filename = $this->_sanitize_file_name( $basic_filename );

		$wp_filetype     = wp_check_filetype( $sanitize_filename );
		$max_upload_size = wp_max_upload_size();

		if ( ! $max_upload_size ) {
			$max_upload_size = 0;
		}

		if ( ( $max_upload_size < $data->get_header( 'content_length' ) )
		     || ! $wp_filetype['ext']
		) {
			$response['id'] = 1;

			return $response;
		}

		if ( 'PUT' === $data->get_method() && ! empty( $data['cms_id'] ) ) {
			$response['id'] = $data['cms_id'];
			if ( 1 === (int) $data['cms_id'] ) {
				if ( isset( get_post( $data['cms_id'] )->post_title )
				     && get_post( $data['cms_id'] )->post_title
				        === $basic_filename
				) {
					$response['cmsEditLink'] = admin_url( 'upload.php?item='
					                                      . $data['cms_id'] );
					$response['cmsOpenLink']
					                         = wp_get_attachment_url( $data['cms_id'] );
				}
			} else {
				$response['cmsEditLink'] = admin_url( 'upload.php?item='
				                                      . $data['cms_id'] );
				$response['cmsOpenLink']
				                         = wp_get_attachment_url( $data['cms_id'] );
			}

			return $response;
		}

		$info = wp_upload_bits( $sanitize_filename, null, $binary_file_string );
		$file = $info['file'];

		$type = wp_check_filetype( $file, null );

		// Scan Media lib by filename/type/md5hash.
		$scan_files = new Desk_Net_Scan_Media_Files();
		$post_title = preg_replace( '/\.[^.]+$/', '', $basic_filename );
		$attach_id  = $scan_files->scan_media_files( $post_title, $type['type'],
			$file );

		if ( false === $attach_id ) {
			$response = $this->load_to_media_lib( $file, $type['type'],
				$info['url'], $basic_filename );

			if ( 'PUT' === $data->get_method() ) {
				if ( empty( $data['post_id'] )
				     || ! is_numeric( $data['post_id'] )
				) {
					$response_data = array(
						'message' => __( 'Wrong file ID', 'desk_net' ),
					);

					return new WP_REST_Response( $response_data, 406 );
				}
			}
		} else {
			wp_delete_file( $info['file'] );

			$response['id']          = $attach_id;
			$response['cmsEditLink']
			                         = admin_url( "upload.php?item=$attach_id" );
			$response['cmsOpenLink'] = $info['url'];
		}

		if ( false != $response ) {
			return $response;
		} else {
			return new WP_REST_Response( '', 500 );
		}
	}

	/**
	 * The replace space in file_name.
	 *
	 * @param string $filename
	 *     The file name.
	 *
	 * @return string $filename
	 */
	public function _sanitize_file_name( $filename ) {
		$filename_raw  = $filename;
		$special_chars = array(
			'?',
			'[',
			']',
			'/',
			'\\',
			'=',
			'<',
			'>',
			':',
			';',
			',',
			'\'',
			'"',
			'&',
			'$',
			'#',
			'*',
			'(',
			')',
			'|',
			'~',
			'`',
			'!',
			'{',
			'}',
			'%',
			'+',
			chr( 0 ),
		);

		/**
		 * Filters the list of characters to remove from a filename.
		 *
		 * @param array  $special_chars
		 *     Characters to remove.
		 * @param string $filename_raw
		 *     Filename as it was passed into _sanitize_file_name().
		 */
		$special_chars = apply_filters( 'sanitize_file_name_chars',
			$special_chars, $filename_raw );
		$filename      = preg_replace( "#\x{00a0}#siu", ' ', $filename );
		$filename      = str_replace( $special_chars, '', $filename );
		$filename      = str_replace( array( '%20', '+' ), '_', $filename );
		$filename      = preg_replace( '/[\r\n\t ]+/', '_', $filename );

		return trim( $filename, '.-_' );
	}

	/**
	 * The detect content mapping.
	 *
	 * @param string $file_id
	 *     The ID on file in media.
	 * @param string $format_id
	 *     The ID for task format.
	 * @param string $content
	 *     The content for text field.
	 *
	 * @return array|string $response
	 */
	public function check_content_matching( $file_id, $format_id, $content = ''
	) {
		$formats_list = get_option( 'wp_desk_net_formats_list' );
		$file         = 'content';

		$content_matching_value = 'body-media';

		// Validating got format id from Desk-Net.
		$format_type = $this->desk_net_validate_format_type( $formats_list,
			$format_id );

		// Exit if format type didn't find.
		if ( false === $format_type ) {
			return $this->post_content;
		}

		switch ( $format_type ) {
			case 1:
				$file_format = 'text';
				break;
			case 2:
				$file_format = 'picture';
				break;
			case 3:
				$file_format = 'video';
				break;
			case 4:
			case 5:
				$file_format = 'other';
				break;
			default:
				$file_format = 'default';
		}

		if ( 'text' !== $file_format ) {
			$file = (string) get_attached_file( $file_id );
		}

		if ( 'default' !== $file_format && 'text' !== $file_format ) {
			$content_matching_value = get_option( 'wp_desk_net_content_'
			                                      . $file_format . '-files' );
		}

		if ( ! $content_matching_value
		     || 'body-media' === $content_matching_value
		) {
			return $this->insert_file_to_content( $file, $file_id, $content );
		} else {
			return $this->post_content;
		}
	}

	/**
	 * Validating format type.
	 *
	 * @param array  $formats_list
	 *     The formats list with types.
	 * @param string $format_id
	 *     The attach file format ID.
	 *
	 * @return string|boolean $format_type
	 */
	public function desk_net_validate_format_type( $formats_list, $format_id ) {
		// Validating the formats list.
		if ( ! $this->desk_net_validate_formats_list( $formats_list ) ) {
			$formats_list = $this->desk_net_get_format_type_from_desk_net();

			if ( $formats_list === false ) {
				return false;
			}
			// Saving new Desk-Net format list.
			update_option( 'wp_desk_net_formats_list', $formats_list );
		}

		// Getting format type ID.
		$formats_list = json_decode( $formats_list, true );

		return $this->desk_net_get_format_type( $formats_list, $format_id );
	}

	/**
	 * Getting format types.
	 *
	 * @param array   $formats_list
	 *     The formats list with types.
	 * @param string  $format_id
	 *     The attach file format ID.
	 * @param boolean $attempt_status
	 *     The attach file format ID.
	 *
	 * @return string|boolean $format_types
	 */
	public function desk_net_get_format_type(
		$formats_list, $format_id, $attempt_status = false
	) {
		// Default format list not valid.
		$format_types = false;

		foreach ( $formats_list as $value ) {
			if ( isset( $value['id'] )
			     && (int) $value['id'] === (int) $format_id
			) {
				$format_types = $value['type'];
			}
		}

		if ( ! $format_types && false === $attempt_status ) {
			// Downloading new Desk-Net format types.
			$formats_list = $this->desk_net_get_format_type_from_desk_net();

			if ( $formats_list !== false ) {
				// Saving new Desk-Net format list.
				update_option( 'wp_desk_net_formats_list', $formats_list );

				$formats_list = json_decode( $formats_list, true );
				// The second attempt getting format type.
				$format_types = $this->desk_net_get_format_type( $formats_list,
					$format_id, true );
			}
		}

		return $format_types;
	}

	/**
	 * Getting format types from Desk-Net by API.
	 *
	 * @return boolean $validate_status
	 */
	public function desk_net_get_format_type_from_desk_net() {
		$request_object = new Desk_Net_Request_Methods();
		$formats_list
		                = $request_object->desk_net_get( Desk_Net_Plugin_Settings::DN_BASE_URL,
			'formats' );

		if ( ! $this->desk_net_validate_formats_list( $formats_list ) ) {
			$formats_list = false;
		}

		return $formats_list;
	}

	/**
	 * Validating format types.
	 *
	 * @param array|string $formats_list
	 *     The formats list with types.
	 *
	 * @return boolean $validate_status
	 */
	public function desk_net_validate_formats_list( $formats_list ) {
		// By default format_list not valid.
		$validate_status = false;
		// Validate string on JSON and encoding to array.
		if ( $this->isJson( $formats_list ) ) {
			$formats_list = json_decode( $formats_list, true );
		}

		$type = gettype( $formats_list );

		if ( $type == 'array' ) {
			// The correct array structure.
			$array_structure = [
				"id"   => '',
				"name" => '',
				"type" => '',
			];
			// Validating array structure.
			$diff_key = array_diff_key( $formats_list[0], $array_structure );

			if ( empty( $diff_key ) ) {
				$validate_status = true;
			}
		}

		return $validate_status;
	}

	/**
	 * The attach content.
	 *
	 * @param string $file
	 *     The link on file.
	 * @param string $file_type
	 *     The type of file.
	 * @param string $url
	 *     The file url.
	 * @param string $filename
	 *     The file name.
	 *
	 * @return array $response
	 */
	public function load_to_media_lib( $file, $file_type, $url, $filename ) {
		$response = [];

		$attachment = array(
			'guid'           => $url,
			'post_mime_type' => $file_type,
			'post_title'     => preg_replace( '/\.[^.]+$/', '', $filename ),
			'post_content'   => '',
			'post_status'    => 'inherit',
		);

		$attach_id = wp_insert_attachment( $attachment, $file );

		$response['id']          = $attach_id;
		$response['cmsEditLink'] = admin_url( "upload.php?item=$attach_id" );
		$response['cmsOpenLink'] = $url;

		// Generate the metadata for the attachment, and update the database record.
		if ( substr_count( $file_type, 'image' ) > 0 ) {
			require_once( ABSPATH . 'wp-admin/includes/image.php' );

			$attach_data = wp_generate_attachment_metadata( $attach_id, $file );

			wp_update_attachment_metadata( $attach_id, $attach_data );
		}

		return $response;
	}

	/**
	 * The insert content to body.
	 *
	 * @param string $file
	 *     The file url.
	 * @param string $file_id
	 *     The file ID.
	 * @param string $content
	 *     The content for text field.
	 *
	 * @return string $response
	 */
	public function insert_file_to_content( $file, $file_id, $content ) {
		if ( ! $this->scan_post_content_on_files_from_DN( $file_id ) ) {
			$this->post_content .= $this->generate_short_code( $file, $file_id,
				$content );
		}

		return $this->post_content;
	}

	/**
	 * Perform detect shortcode supported file type.
	 *
	 * @param $file_type string
	 *                   The type of file.
	 *
	 * @return string
	 */
	public function check_supported_short_code_file_type( $file_type ) {
		$supported_short_code_type = array(
			'mp4',
			'm4v',
			'webm',
			'ogv',
			'wmv',
			'flv',
			'mp3',
			'm4a',
			'ogg',
			'wav',
			'wma',
		);

		foreach ( $supported_short_code_type as $value ) {
			if ( (string) $file_type['ext'] === $value ) {
				return 'short_code';
			}
		}

		return null;
	}

	/**
	 * The scan post content on files/text from Desk-Net.
	 *
	 * @param string $file_id
	 *     The file ID.
	 *
	 * @return bool $response
	 */
	public function scan_post_content_on_files_from_DN( $file_id ) {
		if ( ! empty( $this->post_content ) ) {
			$search_result
				= preg_match( "/\\bdn\\-file\\-id\\-$file_id+[\\'|\\\"|\\:|\\s]/",
				$this->post_content, $old_content );

			if ( ! $search_result ) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}

	/**
	 * The insert content to body.
	 *
	 * @param string $file
	 *     The file url.
	 * @param string $file_id
	 *     The file ID.
	 * @param string $content
	 *     The content for text field.
	 *
	 * @return string $response
	 */
	public function generate_short_code( $file, $file_id, $content ) {
		if ( 'content' !== $file ) {
			$file_type = wp_check_filetype( $file, null );
			$support_short_code
			           = $this->check_supported_short_code_file_type( $file_type );
			preg_match( '/^\w*/', $file_type['type'], $type );

			if ( ! empty( $support_short_code ) ) {
				$content_template = $support_short_code;
			} else {
				$content_template = $type[0];
			}
		} else {
			$content_template = $file;
		}

		switch ( $content_template ) {
			case 'short_code':
				$file_url
					= wp_make_link_relative( wp_get_attachment_url( $file_id ) );
				$short_code
					= "<p>[$type[0] dn-file-id-$file_id:\"$file_id\" src=\"$file_url\"][/$type[0]]</p>";
				break;
			case 'image':
				$image_property = wp_get_attachment_image_src( $file_id,
					'large' );
				$file_url       = wp_make_link_relative( $image_property[0] );
				$short_code
				                = "<p><img class=\"dn-file-id-$file_id\" src=\"$file_url\" alt=\"\" width=\"$image_property[1]\" height=\"$image_property[2]\"/></p>";
				break;
			case 'content':
				$short_code = "<p class=\"dn-file-id-$file_id\">$content</p>";
				break;
			default:
				$file      = get_post( $file_id );
				$file_name = $file->post_title;
				$file_url
				           = wp_make_link_relative( wp_get_attachment_url( $file_id ) );
				$short_code
				           = "<p><a class=\"dn-file-id-$file_id\" href='$file_url'>$file_name</a></p>";
		}

		return $short_code;
	}

	/**
	 * Performing formatting the story data from Desk-Net before insert to post in WP.
	 *
	 * @param array  $data_from_dn
	 *     The post data.
	 * @param string $action
	 *     The action name.
	 *
	 * @return array|object
	 */
	public function transform_received_json_data(
		$data_from_dn, $action = 'create'
	) {
		// Default value.
		$post_data = [];

		if ( empty( $data_from_dn['description'] ) ) {
			$response_data = array(
				'message' => __( 'Error: Story description is empty',
					'desk_net' ),
			);

			return new WP_REST_Response( $response_data, 428 );
		}

		//Getting revision Desk-Net status id.
		$status_id = get_post_meta( $data_from_dn['post_id'],
			'desk_net_status_id', true );

		// Update Status.
		switch ( $action ) {
			case 'create':
				$wp_status_name
					       = get_option( 'wp_desk_net_status_desk_net_to_wp_'
					                     . $data_from_dn['publication']['status']['id'] );
				$wp_status = $this->status_matching( $wp_status_name );

				if ( empty( $wp_status ) ) {
					$wp_status = 'draft';
				}

				$post_data['post_title']
					= $this->get_title_from_desk_net_app( $data_from_dn );

				if ( $this->is_set_desk_net_slug( $data_from_dn ) ) {
					$post_data = $this->set_url_alias( $post_data,
						$data_from_dn['slug'] );
				}

				break;

			case 'update':
				if ( isset( $data_from_dn['publication']['status']['id'] )
				     && (int) $status_id
				        !== (int) $data_from_dn['publication']['status']['id']
				) {
					$wp_status_name
						       = get_option( 'wp_desk_net_status_desk_net_to_wp_'
						                     . $data_from_dn['publication']['status']['id'] );
					$wp_status = $this->status_matching( $wp_status_name );

					if ( empty( $wp_status ) ) {
						$wp_status = 'draft';
					}
				}

				$title = $this->get_title_from_desk_net_app( $data_from_dn,
					'update_post' );
				if ( ! empty( $title ) ) {
					$post_data['post_title'] = $title;
				}

				if ( $this->is_set_desk_net_slug( $data_from_dn )
				     && $this->is_slug_syncing( $data_from_dn['post_id'] )
				) {
					$post_data = $this->set_url_alias( $post_data,
						$data_from_dn['slug'] );
				}

				$post_data['story_description']
					= $this->sanitize_string_from_desk_net( $data_from_dn['description'] );

				break;
		}

		if ( isset( $wp_status ) && ! empty( $wp_status ) ) {
			$post_data['post_status'] = $wp_status;
		}

		if ( isset( $data_from_dn['post_id'] ) ) {
			$post_data['ID'] = $data_from_dn['post_id'];
		}

		if ( ! empty( $data_from_dn['publication']['single']['start'] ) ) {
			$date = $data_from_dn['publication']['single']['start'];
			$date = $this->get_story_date( $date );

			if ( ! $date ) {
				$response_data = array(
					'message' => __( 'Error: wrong data format', 'desk_net' ),
				);

				return new WP_REST_Response( $response_data, 428 );
			} else {
				$post_data = $this->set_post_date( $post_data, $date );

				$post_data['post_modified']     = current_time( 'mysql' );
				$post_data['post_modified_gmt'] = current_time( 'mysql', 1 );
			}
		}

		return $post_data;
	}

	/**
	 * Checking the "Slug" field got from Desk-Net story on data exist.
	 *
	 * @param array $story_data
	 *     The story data.
	 *
	 * @return boolean
	 */
	private function is_set_desk_net_slug( $story_data ) {
		return isset( $story_data['slug'] ) && ! empty( $story_data['slug'] );
	}

	/**
	 * Getting the title value from Desk-Net data.
	 *
	 * @param array  $data_from_dn
	 *     The post data.
	 * @param string $type_action
	 *     The type action with Post element.
	 *
	 * @return string|null
	 */
	public function get_title_from_desk_net_app(
		array $data_from_dn, $type_action = 'create_post'
	) {
		$title = null;
		// Getting first value Headline field.
		$headline = ( isset( $data_from_dn['tasks'] ) )
			? $this->get_headline_field( $data_from_dn ) : null;

		if ( ! empty( $headline ) ) {
			return $headline;
		} else if ( $this->is_set_desk_net_slug( $data_from_dn )
		            && 'title' === $this->get_slug_setting()
		            && $this->is_slug_syncing( $data_from_dn['post_id'] )
		) {
			return $this->sanitize_string_from_desk_net( $data_from_dn['slug'] );
		}

		if ( 'create_post' === $type_action ) {
			$title
				= $this->sanitize_string_from_desk_net( $data_from_dn['description'] );
		}

		return $title;
	}

	/**
	 * Getting Headline field from Desk-Net data.
	 *
	 * @param array $data_from_dn
	 *     The Story data from Desk-Net.
	 *
	 * @return string|null
	 */
	public function get_headline_field( array $data_from_dn ) {
		foreach ( $data_from_dn['tasks'] as $value ) {
			if ( isset( $value['headline'] )
			     && ! empty( $value['headline'] )
			) {
				return $this->sanitize_string_from_desk_net( $value['headline'] );
			}
		}

		return null;
	}

	/**
	 * Perform get slug for post status.
	 *
	 * @param string $wp_status_name
	 *     The WP status name.
	 *
	 * @return string
	 */
	public function status_matching( $wp_status_name ) {
		$post_statuses = get_post_statuses();

		if ( ! empty( $wp_status_name ) && 'No Status' !== $wp_status_name ) {
			foreach ( $post_statuses as $key => $wp_status ) {
				if ( (string) $key === (string) $wp_status_name ) {
					return $key;
				}
			}
		} else {
			return 'draft';
		}
	}

	/**
	 * Getting Slug Syncing settings.
	 *
	 * @return string
	 *   Return Slug Syncing setting value.
	 */
	public function get_slug_setting() {
		$slug_syncing = get_option( 'wp_desk_net_slug_slug_syncing' );

		if ( empty( $slug_syncing ) ) {
			$slug_syncing = 'url_alias';
		}

		return $slug_syncing;
	}

	/**
	 * Sanitize string from Desk-Net fields.
	 *
	 * @param string $string
	 *   The sanitizing string.
	 *
	 * @return string
	 *   Return sanitize string.
	 */
	public function sanitize_string_from_desk_net( $string ) {
		return preg_replace( '/[\r\n\t ]+/', ' ', $string );
	}

	/**
	 * Setting the new URL alias in CMS Post.
	 *
	 * @param array  $post_data
	 *   The post data.
	 * @param string $url_alias
	 *   The new URL alias.
	 *
	 * @return array
	 *   Return updated post array with new URL alias.
	 */
	public function set_url_alias( array $post_data, $url_alias ) {
		if ( 'url_alias' === $this->get_slug_setting() ) {
			// Clearing string for use in URLs.
			$url_alias = sanitize_title( $url_alias );
			// Updating Node alias and off automatic generate URL alias.
			$post_data['post_name'] = $url_alias;
		}

		return $post_data;
	}

	/**
	 * Perform creating post in WP from DN.
	 *
	 * @param array $json_request
	 *     The post params.
	 *
	 * @return WP_REST_Response|boolean
	 */
	public function create_post( $json_request ) {
		if ( $this->active_post_category( $json_request, true ) ) {
			return new WP_REST_Response( '', 200 );
		}

		$lock_post_content = false;
		$post_data
		                   = $this->transform_received_json_data( $json_request );

		if ( empty( $post_data['post_title'] ) ) {
			return new WP_REST_Response( $post_data, 428 );
		}

		if ( $this->is_category_data_exist( $json_request ) ) {
			return false;
		}
		// Setting author data.
		if ( ! empty( $json_request['tasks'] ) ) {
			$user_id
				       = $this->wp_desk_net_get_first_identified_user( $json_request['tasks'] );
			$post_data = ( $user_id )
				? $this->wp_desk_net_set_identified_user( $user_id,
					$json_request['tasks'], $post_data ) : $post_data;
		}

		$post_data['ID'] = ( isset( $post_data['ID'] ) ) ? $post_data['ID']
			: '';

		if ( ! empty( $post_data['ID'] ) ) {
			$content_post       = get_post( $post_data['ID'] );
			$this->post_content = $content_post->post_content;
			$lock_post_content  = get_post_meta( $post_data['ID'],
				'wp_desk_net_lock_update_post_content', true );
		}

		if ( ! empty( $json_request['tasks'] )
		     && true !== (boolean) $lock_post_content
		     && get_post_status( $json_request['post_id'] ) !== 'publish'
		     && get_post_status( $json_request['post_id'] ) !== 'future'
		) {
			foreach ( $json_request['tasks'] as $value ) {
				if ( ! empty( $value['files'] ) ) {
					foreach ( $value['files'] as $file ) {
						$basic_filename = $file['filename'];
						$basic_filename = preg_replace( '/\.[^.]+$/', '',
							$basic_filename );

						if ( isset( get_post( $file['cmsId'] )->post_title )
						     && get_post( $file['cmsId'] )->post_title
						        === $basic_filename
						) {

							$post_data['post_content']
								= $this->check_content_matching( $file['cmsId'],
								$value['format']['id'],
								$this->_sanitize_file_name( $file['filename'] ) );
							// Add task ID for element.
							update_post_meta( $file['cmsId'],
								'wp_desk_net_file_id', $value['id'] );
						}
					}
				} elseif ( ! empty( $value['content'] ) ) {
					$post_data['post_content']
						= $this->check_content_matching( $value['id'],
						$value['format']['id'], $value['content'] );
				}
			}
		}

		$post_data = wp_slash( $post_data );

		if ( strlen( $post_data['post_title'] ) > 77 ) {
			$post_data['post_title'] = mb_substr( $post_data['post_title'], 0,
					77 ) . '...';
		}

		$post_data = apply_filters( 'wp_desk_net_insert_post', $post_data,
			$json_request );
		// Showing correctly date in Gutenberg after creating post by using wp_insert_post.
		add_filter( 'wp_insert_post_data',
			array( $this, 'alter_post_modification_time' ), 99, 2 );
		$post_id = wp_insert_post( $post_data, true );
		remove_filter( 'wp_insert_post_data',
			array( $this, 'alter_post_modification_time' ), 99, 2 );

		// Setting a new category.
		$this->active_post_category( $json_request, false, $post_id );

		// Updating Desk-Net revision.
		Desk_Net_Plugin_Settings::desk_net_set_revision( $post_id,
			$json_request );

		return $this->get_wp_response_data( $post_id );
	}

	/**
	 * Perform updating post on WP from DN.
	 *
	 * @param array $json_request
	 *     The post params.
	 *
	 * @return WP_REST_Response|boolean
	 */
	public function update_post( $json_request ) {
		if ( $this->active_post_category( $json_request, true ) ) {
			return new WP_REST_Response( '', 200 );
		}

		$lock_post_content = false;

		$post_data = $this->transform_received_json_data( $json_request,
			'update' );

		if ( empty( $post_data['story_description'] ) ) {
			return new WP_REST_Response( 'Empty story description.', 428 );
		}

		// Checking on exist category id in JSON data.
		if ( $this->is_category_data_exist( $json_request ) ) {
			return false;
		}
		// Setting author data.
		if ( ! empty( $json_request['tasks'] ) ) {
			$user_id
				       = $this->wp_desk_net_get_first_identified_user( $json_request['tasks'] );
			$post_data = ( $user_id )
				? $this->wp_desk_net_set_identified_user( $user_id,
					$json_request['tasks'], $post_data ) : $post_data;
		}

		$post_data['ID'] = ( isset( $post_data['ID'] ) ) ? $post_data['ID']
			: '';

		if ( ! empty( $post_data['ID'] ) ) {
			$content_post       = get_post( $post_data['ID'] );
			$this->post_content = $content_post->post_content;
			$lock_post_content  = get_post_meta( $post_data['ID'],
				'wp_desk_net_lock_update_post_content', true );
		}

		if ( ! empty( $json_request['tasks'] )
		     && true !== (boolean) $lock_post_content
		     && get_post_status( $json_request['post_id'] ) !== 'publish'
		     && get_post_status( $json_request['post_id'] ) !== 'future'
		) {
			foreach ( $json_request['tasks'] as $value ) {
				if ( ! empty( $value['files'] ) ) {
					foreach ( $value['files'] as $file ) {
						$basic_filename = $file['filename'];
						$basic_filename = preg_replace( '/\.[^.]+$/', '',
							$basic_filename );

						if ( isset( get_post( $file['cmsId'] )->post_title )
						     && get_post( $file['cmsId'] )->post_title
						        === $basic_filename
						) {
							$post_data['post_content']
								= $this->check_content_matching( $file['cmsId'],
								$value['format']['id'],
								$this->_sanitize_file_name( $file['filename'] ) );
							// Add task ID for element.
							update_post_meta( $file['cmsId'],
								'wp_desk_net_file_id', $value['id'] );
						}
					}
				} elseif ( ! empty( $value['content'] ) ) {
					$post_data['post_content']
						= $this->check_content_matching( $value['id'],
						$value['format']['id'], $value['content'] );
				}
			}
		}

		update_post_meta( $post_data['ID'], 'desk_net_description',
			html_entity_decode( $post_data['story_description'] ) );
		unset( $post_data['story_description'] );

		if ( ! empty( $post_data['post_date_gmt'] ) ) {
			$date                       = strtotime( $post_data['post_date_gmt'] );
			$post_data['post_date_gmt'] = date( 'Y-m-d H:i:s', $date );
			$add_timezone
			                            = strtotime( $post_data['post_date_gmt'] )
			                              + get_option( 'gmt_offset' ) * 60
			                                * 60;
			$post_data['post_date']     = date( 'Y-m-d H:i:s', $add_timezone );
		} else {
			$date                       = strtotime( get_the_date( 'Y-m-d H:i:s',
				$post_data['ID'] ) );
			$post_data['post_date_gmt'] = get_gmt_from_date( $date,
				'Y-m-d H:i:s' );
			$post_data['post_date']     = date( 'Y-m-d H:i:s', $date );
		}

		$post_data = wp_slash( $post_data );
		$post_data = apply_filters( 'wp_desk_net_update_post', $post_data,
			$json_request );
		$post_id   = wp_update_post( $post_data );

		// Setting a new category.
		$this->active_post_category( $json_request, false, $post_id );
		// Updating Desk-Net revision.
		Desk_Net_Plugin_Settings::desk_net_set_revision( $post_id,
			$json_request );

		return $this->get_wp_response_data( $post_id );
	}

	/**
	 * Perform update Active Category for post on WP from DN.
	 *
	 * @param array   $json_request
	 *     The Desk-Net data.
	 * @param integer $post_id
	 *     The WP post ID.
	 * @param boolean $detect_import
	 *     The detect Category 'Do Not Import'.
	 *
	 * @return bool|void
	 */
	public function active_post_category(
		$json_request, $detect_import = false, $post_id = null
	) {
		if ( ! empty( $json_request['publication']['category']['id'] )
		     && isset( $json_request['publication']['category']['name'] )
		) {
			$category_id = $json_request['publication']['category']['id'];

			if ( $detect_import ) {
				return $this->check_do_not_import( get_option( 'wp_desk_net_category_desk_net_to_wp_'
				                                               . $category_id ) );
			}

			$args = array(
				'taxonomy'   => 'category',
				'hide_empty' => false,
			);

			$category_list    = get_terms( $args );
			$mapping_category = false;

			foreach ( $category_list as $key => $value ) {
				if ( (int) get_option( 'wp_desk_net_category_desk_net_to_wp_'
				                       . $category_id )
				     === (int) $value->term_id
				) {
					wp_set_post_categories( $post_id,
						$value->term_id, false );
					$mapping_category = true;
				}
			}

			if ( ! $mapping_category ) {
				wp_set_post_categories( $post_id );
			}
		} elseif ( ! isset( $json_request['publication']['category']['name'] )
		           && ! empty( get_option( 'wp_desk_net_category_desk_net_to_wp_no_category' ) )
		) {
			$no_category
				= get_option( 'wp_desk_net_category_desk_net_to_wp_no_category' );

			if ( $detect_import ) {
				return $this->check_do_not_import( $no_category );
			}

			if ( 'no_category' === $no_category ) {
				wp_set_post_terms( $post_id, array(), 'category', false );
			} else {
				wp_set_post_categories( $post_id, $no_category, false );
			}
		}
	}

	/**
	 * Perform scan value on 'do_not_import' or 'Do not import'.
	 *
	 * @param string $value
	 *
	 * @return boolean
	 */
	public function check_do_not_import( $value ) {
		if ( 'do_not_import' === $value || 'Do not import' === $value ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Perform validate user from Desk-Net by email.
	 *
	 * @param string $email
	 *     The email Desk-Net user.
	 *
	 * @return null|integer $response
	 */
	public function valid_user( $email ) {
		include_once( ABSPATH . 'wp-includes/pluggable.php' );

		$user = get_user_by( 'email', $email );

		if ( $user ) {
			return $user->ID;
		}

		return null;
	}

	/**
	 * Perform authorization by API.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return object $response
	 */
	public function auth( WP_REST_Request $data ) {
		$authorization      = $this->get_request_token( $data );
		$request_api_key    = $data->get_param( 'api_key' );
		$request_api_secret = $data->get_param( 'api_secret' );
		// Response Message for case when System got not valid credentials.
		$response_data = array(
			'message' => __( 'Wrong credentials', 'desk_net' ),
		);

		if ( null !== $authorization ) {
			// Decoding the authorization string.
			$user_data = explode( ':', base64_decode( $authorization ) );

			if ( get_option( 'wp_desk_net_api_key' ) === (string) $user_data[0]
			     && get_option( 'wp_desk_net_api_secret' )
			        === (string) $user_data[1]
			) {
				$response = $this->desk_net_auth_response();
			} else {
				$response = new WP_REST_Response( $response_data, 401 );
			}
		} elseif ( null !== $request_api_key && null !== $request_api_secret
		           && get_option( 'wp_desk_net_api_key' ) === $request_api_key
		           && get_option( 'wp_desk_net_api_secret' )
		              === $request_api_secret
		) {
			$response = $this->desk_net_auth_response();
		} else {
			$response = new WP_REST_Response( $response_data, 401 );
		}

		return $response;
	}

	/**
	 * Perform response data.
	 *
	 * @return object $response
	 */
	public function desk_net_auth_response() {
		$response_data = array(
			'access_token' => $this->generate_token(),
			'token_type'   => 'bearer',
			'expires_in'   => 943199,
		);

		$response = new WP_REST_Response( $response_data, 200 );
		$response->header( 'Access-Control-Allow-Origin',
			apply_filters( 'wp_desk_net_access_control_allow_origin', '*' ) );

		return $response;
	}

	/**
	 * Perform join style to page "WordPress Credentials".
	 *
	 */
	public function desc_net_css_js() {
		wp_enqueue_style( 'style-name',
			plugin_dir_url( __FILE__ ) . 'css/desk_net.css' );
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'js-name',
			plugin_dir_url( __FILE__ ) . 'js/desk_net.js', array( 'jquery' ) );
	}

	/**
	 * Perform update activate and deactivate status list. Update active
	 * statuses by triggersExport.
	 *
	 * @param array $status_list
	 *     The default status list.
	 *
	 * @return array $status_list
	 *     The refreshed status list.
	 */
	public function check_triggers_export_status( $status_list ) {
		if ( empty( $status_list['deactivatedStatuses'] ) ) {
			$status_list['deactivatedStatuses'] = array();
		}

		foreach ( $status_list['activeStatuses'] as $key => $value ) {
			if ( false
			     === $status_list['activeStatuses'][ $key ]['triggersExport']
			) {
				array_unshift( $status_list['deactivatedStatuses'],
					$status_list['activeStatuses'][ $key ] );
				unset( $status_list['activeStatuses'][ $key ] );
			}
		}

		return $status_list;
	}

	/**
	 * Perform generate post link.
	 *
	 * @param string $url
	 *     The post url.
	 * @param object $post
	 *     The post.
	 *
	 * @return string
	 */
	public function desk_net_post_link( $url, $post ) {
		$desk_net_id = get_query_var( 'desk_net_id' );

		if ( 'post' === $post->post_type && empty( $desk_net_id ) ) {
			$story_id = get_post_meta( $post->ID, 'story_id' );

			if ( ! empty( $story_id[0] ) ) {

				$base_url = get_site_url();

				$url = $base_url . '/' . $story_id[0] . '/' . $post->post_name
				       . '/';
			}
		}

		return $url;
	}

	/**
	 * Perform generate WP token.
	 *
	 * @return string $status_list
	 */
	public function generate_token() {
		$characters        = '0123456789abcdefghijklmnopqrstuvwxyz';
		$characters_length = strlen( $characters );
		$lengths           = array( 8, 4, 4, 4, 12 );
		$token             = '';

		foreach ( $lengths as $key => $length ) {
			for ( $i = 0; $i < $length; $i ++ ) {
				$token .= $characters[ rand( 0, $characters_length - 1 ) ];
			}

			if ( 4 !== (int) $key ) {
				$token .= '-';
			}
		}

		update_option( 'wp_desk_net_auth_token', $token );

		return $token;
	}

	/**
	 * Perform validate token.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return boolean
	 */
	public function check_token( $data ) {
		$api_token = get_option( 'wp_desk_net_auth_token' );

		$request_token = $this->get_request_token( $data );

		if ( null === $request_token ) {
			return false;
		}

		if ( ! empty( $api_token ) && ! empty( $request_token )
		     && $api_token === $request_token
		) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Getting request token.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return string
	 */
	public function get_request_token( $data ) {
		$authorization_header = $this->get_authorization_header( $data );

		if ( null === $authorization_header ) {
			return null;
		}

		$authorization = preg_split( '/^(Basic|Bearer)+\s/',
			$authorization_header );

		return ( isset( $authorization[1] ) ? $authorization[1]
			: null );
	}

	/**
	 * Getting authorization header.
	 *
	 * @param WP_REST_Request $data
	 *     The data from Desk-Net.
	 *
	 * @return string
	 */
	public function get_authorization_header( $data ) {
		if ( function_exists( 'getallheaders' ) ) {
			$all_headers = getallheaders();
			$all_headers = array_change_key_case( $all_headers, CASE_LOWER );

			if ( array_key_exists( 'authorization', $all_headers ) ) {
				$authorization_header = $all_headers['authorization'];

				return ( ! empty( $authorization_header )
					? $authorization_header : null );
			}
		}

		$authorization_header = $data->get_header( 'authorization' );

		return ( ! empty( $authorization_header ) ? $authorization_header
			: null );
	}

	/**
	 * Perform getting WP user ID by email from Desk-Net.
	 *
	 * @param array $tasks
	 *     The story params.
	 *
	 * @return null|integer
	 */
	private function wp_desk_net_get_first_identified_user( $tasks ) {
		foreach ( $tasks as $task ) {
			if ( $this->is_user_data_exist( $task ) ) {
				$user_id = $this->valid_user( $task['assignee']['name'] );
				if ( $user_id ) {
					return $user_id;
				}
			}
		}

		return null;
	}

	/**
	 * Perform setting Desk-Net author ID to WP user.
	 *
	 * @param integer $user_id
	 *     The user ID in WP.
	 * @param array   $story_data
	 *     The story params.
	 * @param array   $post_data
	 *     The post params.
	 *
	 * @return array
	 */
	private function wp_desk_net_set_identified_user(
		$user_id, $story_data, $post_data
	) {
		$post_data['post_author'] = $user_id;
		// Detecting WordPress VIP server.
		if ( defined( 'WPCOM_IS_VIP_ENV' ) && true === WPCOM_IS_VIP_ENV ) {
			// Save author ID.
			update_user_attribute( $user_id, 'desk_net_author_id',
				$story_data[0]['assignee']['id'] );
		} else {
			// Saving author ID.
			// @codingStandardsIgnoreStart.
			update_user_meta( $user_id, 'desk_net_author_id',
				$story_data[0]['assignee']['id'] );
			// @codingStandardsIgnoreEnd.
		}

		return $post_data;
	}

	/**
	 * Checking author data to exist in the story data.
	 *
	 * @param array $task
	 *     The story params.
	 *
	 * @return boolean
	 */
	private function is_user_data_exist( array $task ) {
		$options_list = [
			[ 'format', 'name' ],
			[ 'assignee' ]
		];
		foreach ( $options_list as $value ) {
			$is_user_data = $this->array_multi_key_exists( $value, $task );
		}

		return $is_user_data;
	}

	/**
	 * Checking keys to exist in the array.
	 *
	 * @param array $arrNeedles
	 *     The list with keys.
	 * @param array $arrHaystack
	 *     The array for checking.
	 *
	 * @return boolean
	 */
	public function array_multi_key_exists(
		array $arrNeedles, array $arrHaystack
	) {
		$Needle = array_shift( $arrNeedles );
		if ( 0 == count( $arrNeedles ) ) {
			return ( array_key_exists( $Needle, $arrHaystack )
			         && ! is_null( $arrHaystack[ $Needle ] ) );
		} else {
			if ( ! array_key_exists( $Needle, $arrHaystack ) ) {
				return false;
			} else {
				return $this->array_multi_key_exists( $arrNeedles,
					$arrHaystack[ $Needle ] );
			}
		}
	}

	/**
	 * Checking the category data on exist in the story data.
	 *
	 * @param array $story_data
	 *     The story params.
	 *
	 * @return boolean
	 */
	private function is_category_data_exist( array $story_data ) {
		if ( $this->array_multi_key_exists( [ 'publication', 'category', 'id' ],
			$story_data )
		) {
			$category_id = $story_data['publication']['category']['id'];
			if ( get_option( 'wp_desk_net_category_desk_net_to_wp_'
			                 . $category_id ) === 'Do not import'
			) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Validating string on JSON.
	 *
	 * @param string $string
	 *     The validating string.
	 *
	 * @return bool
	 */
	public function isJson( $string ) {
		json_decode( $string );

		return ( json_last_error() == JSON_ERROR_NONE );
	}

	/**
	 * Importing resources for Gutenberg editor.
	 *
	 */
	public function desk_net_enqueue() {
		if ( is_plugin_active( 'classic-editor/classic-editor.php' ) ) {
			return;
		}
		wp_enqueue_script(
			'desk-net-gutenberg-script',
			plugins_url( 'js/gutenberg_desk_net_panel.js', __FILE__ ),
			array( 'wp-editor', 'wp-i18n', 'wp-edit-post', 'wp-data' )
		);
		wp_enqueue_style(
			'desk-net-gutenberg-style',
			plugins_url( 'css/gutenberg_editor.css', __FILE__ )
		);
		wp_localize_script(
			'desk-net-gutenberg-script',
			'desk_net',
			array(
				'icon_src'         => plugins_url( 'img/favicon.ico',
					__FILE__ ),
				'view_in_desk_net' => esc_url( Desk_Net_Plugin_Settings::DN_BASE_URL
				                               . '/mySchedulePage.htm?fragment=de' ),
			)
		);
	}

	/**
	 * Getting story description data for current post.
	 *
	 * @param object $post
	 *     The current post.
	 *
	 * @return string
	 */
	public function get_desk_net_story_description( $post ) {
		$desk_net_description = esc_html( get_post_meta( $post->ID,
			'desk_net_description', true ) );

		return $this->story_data_removed_status( $post )
		       . $desk_net_description;
	}

	/**
	 * Checking story on exist in Desk-Net app.
	 *
	 * @param object $post
	 *     The current post.
	 *
	 * @return string
	 */
	public function story_data_removed_status( $post ) {
		$html = '';
		if ( get_post_meta( $post->ID, 'wp_desk_net_remove_status', true )
		     === 'removed'
		) {
			$html .= esc_html__( 'Deleted/removed – ', 'desk_net' );
		}

		return $html;
	}

	/**
	 * Getting all story data for current post.
	 *
	 * @param object $post
	 *     The current post.
	 *
	 * @return array|boolean
	 */
	public function get_desk_net_post_data( $post ) {
		$data['desk_net_id']          = esc_html( get_post_meta( $post->ID,
			'story_id', true ) );
		$data['desk_net_status_name'] = esc_html( get_post_meta( $post->ID,
			'desk_net_status_name', true ) );
		$data['desk_net_description'] = esc_html( get_post_meta( $post->ID,
			'desk_net_description', true ) );
		if ( ! empty( $data['desk_net_id'] )
		     && ! empty( $data['desk_net_description'] )
		) {
			return $data;
		}

		return false;
	}

	/**
	 * Setting custom post_modified date on the wp_insert_post action.
	 *
	 * @param array $post                   The current post.
	 * @param array $postarr                {
	 *                                      An array of elements that make up a post to update or insert.
	 *
	 * @type string $post_modified          The date when the post was last modified. Default is
	 *                                         the current time.
	 * @type string $post_modified_gmt      The date when the post was last modified in the GMT
	 * }
	 * @return array
	 *     The post ID on success.
	 */
	function alter_post_modification_time( $post, $postarr ) {
		if ( ! empty( $postarr['post_modified'] )
		     && ! empty( $postarr['post_modified_gmt'] )
		) {
			$post['post_modified']     = $postarr['post_modified'];
			$post['post_modified_gmt'] = $postarr['post_modified_gmt'];
		}

		return $post;
	}
}

// Removing all Desk-Net data.
register_uninstall_hook( 'desk-net/desk-net.php',
	'desk_net_delete_all_option' );

/**
 * Perform delete fields with prefix from WordPress DB.
 *
 * @param string $prefix
 *   The field prefix.
 */
function desk_net_delete_all_option( $prefix = 'wp_desk_net_' ) {
	global $wpdb;
	// @codingStandardsIgnoreStart.
	$plugin_options
		= $wpdb->get_results( $wpdb->prepare( "SELECT option_name FROM $wpdb->options WHERE option_name LIKE %s",
		'wp_desk_net_%' ) );
	// @codingStandardsIgnoreEnd.
	foreach ( $plugin_options as $option ) {
		delete_option( $option->option_name );
	}
}

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly.

new WP_Desk_Net;
