<?php

/**
 * @package Desk-Net Core.
 * @author  Desk-Net GmbH.
 */

require_once plugin_dir_path( __FILE__ )
             . 'template/desk-net-template-matching-page.php';
include_once plugin_dir_path( __FILE__ )
             . 'template/class-desk-net-common-components.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-delete-methods.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-request-methods.php';
include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-plugin-settings.php';

class Desk_Net_Core {
	/**
	 * @var int Error Message.
	 *
	 * Default value 1 - Successful update.
	 */
	public $error_message = 1;
	/**
	 * @var string Token.
	 */
	protected $token;
	/**
	 * @var array Warning Message.
	 */
	protected $warning_message = array();
	/**
	 * @var bool Status Upload page.
	 */
	protected $status_upload = false;
	/**
	 * Holds the values to be used in the fields callbacks
	 */
	private $options;

	function __construct() {
		register_activation_hook( 'desk-net/desk-net.php', array(
			$this,
			'wp_desk_net_activate',
		) );

		add_action( 'admin_menu', array( $this, 'desk_net_admin_menu' ) );
		add_action( 'admin_init', array( $this, 'desk_net_settings_init' ) );
		add_action( 'edit_form_after_title', array(
			$this,
			'desk_net_description',
		) );

		add_filter( 'plugin_action_links_' . 'desk-net/desk-net.php', array(
			$this,
			'add_action_links',
		) );
	}

	/**
	 * Initialize admin menu.
	 */
	function desk_net_admin_menu() {
		add_menu_page(
			__( 'Desk-Net', 'desk_net' ),
			__( 'Desk-Net', 'desk_net' ),
			'manage_options',
			'wp-desk-net',
			array( $this, 'desk_net_settings_page' ),
			'dashicons-arrow-right-alt2',
			6
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'WordPress Credentials', 'desk_net' ),
			__( 'WordPress Credentials', 'desk_net' ),
			'manage_options',
			'wp-desk-net'
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Desk-Net Credentials', 'desk_net' ),
			__( 'Desk-Net Credentials', 'desk_net' ),
			'manage_options',
			'wp_desk_net_credential',
			array( $this, 'desk_net_credential_page' )
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Status Matching', 'desk_net' ),
			__( 'Status Matching', 'desk_net' ),
			'manage_options',
			'wp_desk_net_status-matching',
			array( $this, 'desk_net_status_matching_page' )
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Category Matching', 'desk_net' ),
			__( 'Category Matching', 'desk_net' ),
			'manage_options',
			'wp_desk_net_category-matching',
			array( $this, 'desk_net_category_matching_page' )
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Content Settings', 'desk_net' ),
			__( 'Content Settings', 'desk_net' ),
			'manage_options',
			'wp_desk_net_content-matching',
			array( $this, 'content_matching_page' )
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Slug Syncing', 'desk_net' ),
			__( 'Slug Syncing', 'desk_net' ),
			'manage_options',
			'desk-net-slug',
			array( $this, 'create_desk_net_slug_page' )
		);

		add_submenu_page(
			'wp-desk-net',
			__( 'Desk-Net ID in URL', 'desk_net' ),
			__( 'Desk-Net ID in URL', 'desk_net' ),
			'manage_options',
			'wp_desk_net_permalinks',
			array( $this, 'permalinks_page' )
		);
	}

	/**
	 * Perform template settings page.
	 */
	function desk_net_settings_page() {

		echo '<div class="wrap">';

		settings_errors();

		echo '<form action="options.php" method="post" id="generate-credentials">';

		settings_fields( 'desk_net_settings' );
		do_settings_sections( 'desk_net_settings' );
		submit_button( 'Generate new credentials', 'secondary' );

		echo '<p><input type="button" name="submitForm" id="submitForm" class="button" value="Generate new credentials"></p>';
		echo '</form>';
		// Modal page.
		echo '<div id="consentModal" class="modal">
                <!-- Modal content -->
                <div class="modal-content">
                    <span class="close">&times;</span>
                    <p>'
		     . esc_html__( 'Are you sure to generate new credentials?',
				'desk_net' ) . '</p>
                    <div class="position-control-element"><input type="button" name="confirm" 
                    id="confirm" class="button button-primary" value="'
		     . esc_attr__( 'Confirm', 'desk_net' ) . '">
                    <input type="button" name="cancel" id="cancel" class="button" value="'
		     . esc_attr__( 'Cancel', 'desk_net' ) . '"></div>
                </div>

              </div>';
		echo '</div>';
	}

	/**
	 * Perform template credential page.
	 */
	function desk_net_credential_page() {
		echo '<div class="wrap">';

		settings_errors();

		echo '<form action="options.php" method="post">';

		settings_fields( 'desk_net_authorization' );
		do_settings_sections( 'desk_net_authorization' );
		submit_button( 'Save' );

		echo '</form>';

		echo '</div>';
	}

	/**
	 * Perform template status matching page.
	 */
	function desk_net_status_matching_page() {
		echo '<div class="wrap page-matching">';

		settings_errors();
		$status_list = get_option( 'wp_desk_net_desk-net-list-active-status' );

		if ( ! empty( $status_list ) && ! isset( $status_list['error'] ) ) {
			echo '<form action="options.php" method="post">';

			settings_fields( 'wp_desk_net_status_matching' );
			$this->custom_do_settings_sections( 'wp_desk_net_status_matching' );
			submit_button( 'Save' );

			echo '</form>';
		} else {
			echo '<h2>'
			     . esc_html__( 'There is no connection from Desk-Net to WordPress. Code: 08',
					'desk_net' ) . '</h2>';
		}
		echo '</div>';
	}

	/**
	 * Perform generate custom template for Matching page.
	 *
	 * @param string $page
	 *   The page name.
	 * @param array  $css_classes
	 *   The additional CSS classes list.
	 * @param array  $fields_list_with_css_classes
	 *   The fields list containing field _id which the additional CSS classes must be added.
	 */
	function custom_do_settings_sections(
		$page, array $css_classes = [ 'label' ],
		array $fields_list_with_css_classes = []
	) {
		global $wp_settings_sections, $wp_settings_fields;

		if ( ! isset( $wp_settings_sections )
		     || ! isset( $wp_settings_sections[ $page ] )
		) {
			return;
		}

		foreach ( (array) $wp_settings_sections[ $page ] as $section ) {
			echo '<h3>' . esc_html( $section['title'] ) . '</h3>';
			call_user_func( $section['callback'], $section );

			if ( ! isset( $wp_settings_fields )
			     || ! isset( $wp_settings_fields[ $page ] )
			     || ! isset( $wp_settings_fields[ $page ][ $section['id'] ] )
			) {
				continue;
			}
			echo '<div class="settings-form-wrapper">';
			$this->custom_do_settings_fields( $page, $section['id'],
				$css_classes, $fields_list_with_css_classes );
			echo '</div>';
		}
	}

	/**
	 * Perform generate custom settings fields for Matching page.
	 *
	 * @param string $page
	 * @param string $section
	 * @param array  $css_classes
	 *   The additional CSS classes list.
	 * @param array  $fields_list_with_css_classes
	 *   The fields list containing field _id which the additional CSS classes must be added.
	 */
	function custom_do_settings_fields(
		$page, $section, array $css_classes, array $fields_list_with_css_classes
	) {
		global $wp_settings_fields;

		if ( ! isset( $wp_settings_fields )
		     || ! isset( $wp_settings_fields[ $page ] )
		     || ! isset( $wp_settings_fields[ $page ][ $section ] )
		) {
			return;
		}

		foreach ( (array) $wp_settings_fields[ $page ][ $section ] as $field ) {
			echo '<div class="settings-form-row">';

			if ( ! empty( $field['title'] ) ) {
				if ( ! empty( $field['args']['label_for'] ) ) {
					echo '<p class="' . implode( ' ', $css_classes ) . '">
                    <label for="' . esc_attr( $field['args']['label_for'] ) . '">
                    ' . esc_html( $field['title'] ) . '</label><br />';
				} else {
					echo '<p class="' . implode( ' ', $css_classes ) . '">'
					     . esc_html( $field['title'] ) . '<br />';
				}
				call_user_func( $field['callback'], $field['args'] );
				echo '</p>';
			} elseif ( in_array( $field['id'],
				$fields_list_with_css_classes )
			) {
				echo '<div class="' . implode( ' ', $css_classes ) . '">';
				call_user_func( $field['callback'], $field['args'] );
				echo '</div>';
			} else {
				call_user_func( $field['callback'], $field['args'] );
			}

			echo '</div>';
		}
	}

	/**
	 * Perform template category matching page.
	 */
	function desk_net_category_matching_page() {
		$platform_id = get_option( 'wp_desk_net_platform_id' );
		$token       = false;
		echo '<div class="wrap page-matching">';

		settings_errors();

		if ( ! empty( get_option( 'wp_desk_net_user_password' ) )
		     && ! empty( get_option( 'wp_desk_net_user_login' ) )
		     && 'not_valid' !== get_option( 'wp_desk_net_token' )
		) {
			$token
				= $this->desk_net_check_validate_token( get_option( 'wp_desk_net_user_login' ),
				get_option( 'wp_desk_net_user_password' ) );
		}

		if ( false !== $token ) {
			if ( ! empty( $platform_id ) ) {
				$this->desk_net_update_category_list();
				$category_list
					= get_option( 'wp_desk_net_desk_net_category_list' );

				if ( ! empty( $category_list )
				     && ! isset( $category_list['message'] )
				) {
					echo '<form action="options.php" method="post" autocomplete="off">';

					settings_fields( 'wp_desk_net_category_matching' );
					$this->custom_do_settings_sections( 'wp_desk_net_category_matching' );
					submit_button( 'Save' );

					echo '</form>';
				} else {
					echo '<h2>'
					     . esc_html__( 'There are no categories entered in Desk-Net.',
							'desk_net' ) . '</h2>';
				}
			} else {
				echo '<h2>'
				     . esc_html__( 'There is no connection from Desk-Net to WordPress. Code: 08',
						'desk_net' ) . '</h2>';
			}
		} else {
			echo '<h2>'
			     . esc_html__( 'There is no connection from WordPress to Desk-Net. Code: 07',
					'desk_net' ) . '</h2>';
		}
		echo '</div>';
	}

	/**
	 * Perform validate token.
	 *
	 * @param string $login
	 *   The Desk-Net user login.
	 * @param string $password
	 *   The Desk-Net user password.
	 *
	 * @return string
	 */
	public function desk_net_check_validate_token( $login, $password ) {
		$request_obj = new Desk_Net_Request_Methods();
		$token       = $request_obj->desk_net_get_token( $login, $password );

		return $token;
	}

	/**
	 * Perform Update Category List.
	 */
	public function desk_net_update_category_list() {
		$platform_id    = get_option( 'wp_desk_net_platform_id' );
		$save_category_list_for_platform
		                = get_option( 'wp_desk_net_desk_net_category_list' );
		$request_method = new Desk_Net_Request_Methods();

		$category_list_for_platform
			= json_decode( $request_method->desk_net_get( Desk_Net_Plugin_Settings::DN_BASE_URL,
			'categories/platform', $platform_id ), true );
		if ( empty( $category_list_for_platform )
		     || isset( $category_list_for_platform['message'] )
		) {
			delete_option( 'wp_desk_net_desk_net_category_list' );

			return;
		}

		if ( ! empty( $save_category_list_for_platform ) ) {
			$element_list     = array();
			$wp_category_list = get_terms( 'category',
				'orderby=namet&order=ASC&hide_empty=0' );
			$wp_category_list
			                  = json_decode( wp_json_encode( $wp_category_list ),
				true );
			foreach ( $wp_category_list as $value ) {
				$element_list[] = $value['term_id'];
			}
			$delete_elements = new Desk_Net_Delete_Methods();
			$delete_elements->desk_net_shape_deleted_items( $category_list_for_platform,
				$save_category_list_for_platform, $element_list, 'category' );
		}

		update_option( 'wp_desk_net_desk_net_category_list',
			$category_list_for_platform );
	}

	/**
	 * Create 'Slug syncing' page.
	 */
	function create_desk_net_slug_page() {
		$this->options = get_option( 'wp_desk_net_slug_list' );

		echo '<div class="wrap page-matching slug">';

		settings_errors();

		echo '<form action="options.php" method="post">';

		settings_fields( 'desk_net_slug_option' );
		$this->custom_do_settings_sections(
			'desk_net_slug_syncing',
			[ 'dn-d-flex', 'dn-width-152', 'dn-flex-align-items-center' ],
			[ 'desk_net_initial_slug_syncing' ] );
		submit_button( 'Save' );

		echo '</form>';
		echo '</div>';
	}

	/**
	 * Perform template content matching page.
	 */
	function content_matching_page() {
		echo '<div class="wrap page-matching">';

		settings_errors();

		echo '<form action="options.php" method="post">';

		settings_fields( 'wp_desk_net_content_matching' );
		$this->custom_do_settings_sections( 'wp_desk_net_content_matching' );

		esc_html_e( 'Text will always be imported right into the post’s body.',
			'desk_net' );

		submit_button( 'Save' );

		echo '</form>';

		echo '</div>';

	}

	/**
	 * Perform template permalinks page.
	 */
	function permalinks_page() {

		echo '<div class="wrap">';

		settings_errors();

		echo '<form action="options.php" method="post">';

		settings_fields( 'wp_desk_net_permalinks' );
		do_settings_sections( 'wp_desk_net_permalinks' );
		submit_button( 'Save' );

		echo '</form>';
		echo '</div>';
	}

	/**
	 * Perform settings fields for plugin pages.
	 */
	function desk_net_settings_init() {
		// The "WordPress Credentials" page.
		register_setting( 'desk_net_settings', 'wp_desk_net__settings', array(
			$this,
			'generate_api_credentials',
		) );

		add_settings_section(
			'wp_desk_net_settings',
			__( 'WordPress Credentials', 'desk_net' ),
			array( $this, 'desk_net_settings_section_html' ),
			'desk_net_settings'
		);

		add_settings_field(
			'wp_desk_net_api_url',
			__( 'API URL', 'desk_net' ),
			array( $this, 'api_url_html' ),
			'desk_net_settings',
			'wp_desk_net_settings'
		);

		add_settings_field(
			'wp_desk_net_api_key',
			__( 'API User', 'desk_net' ),
			array( $this, 'api_key_input_html' ),
			'desk_net_settings',
			'wp_desk_net_settings'
		);

		add_settings_field(
			'wp_desk_net_api_secret',
			__( 'API Secret', 'desk_net' ),
			array( $this, 'api_secret_input_html' ),
			'desk_net_settings',
			'wp_desk_net_settings'
		);

		// The "Desk-Net Credentials" page.
		register_setting( 'desk_net_authorization',
			'wp_desk_net__authorization', array(
				$this,
				'desk_net_save_authorization',
			) );

		add_settings_section(
			'wp_desk_net__authorization',
			__( 'Desk-Net Credentials', 'desk_net' ),
			array( $this, 'authorization_section_html' ),
			'desk_net_authorization'
		);

		add_settings_field(
			'wp_desk_net_user_login',
			__( 'Desk-Net API Login', 'desk_net' ),
			array( $this, 'wp_desk_net_login_fidel_html' ),
			'desk_net_authorization',
			'wp_desk_net__authorization'
		);

		add_settings_field(
			'wp_desk_net_user_password',
			__( 'Desk-Net API Password', 'desk_net' ),
			array( $this, 'wp_desk_net_password_fidel_html' ),
			'desk_net_authorization',
			'wp_desk_net__authorization'
		);

		// The "Status Matching" page.
		register_setting( 'wp_desk_net_status_matching',
			'wp_desk_net_status_list', array(
				$this,
				'desk_net_save_matching_settings',
			) );

		add_settings_section(
			'wp_desk_net_status_list',
			__( 'Status Matching', 'desk_net' ),
			array( $this, 'desk_net_status_matching_html' ),
			'wp_desk_net_status_matching'
		);

		add_settings_field(
			'wp_desk_net_desk_net_to_wp_status',
			__( 'Desk-Net to WordPress', 'desk_net' ),
			array( $this, 'desk_net_to_wp_status_html' ),
			'wp_desk_net_status_matching',
			'wp_desk_net_status_list'
		);

		add_settings_field(
			'wp_desk_net_wp_to_desk_net_status',
			__( 'WordPress to Desk-Net', 'desk_net' ),
			array( $this, 'wp_to_desk_net_status_html' ),
			'wp_desk_net_status_matching',
			'wp_desk_net_status_list'
		);

		// The "Category Matching" page.
		register_setting( 'wp_desk_net_category_matching',
			'wp_desk_net_category_list', array(
				$this,
				'desk_net_save_matching_settings',
			) );

		add_settings_section(
			'wp_desk_net_category_list',
			__( 'Category Matching', 'desk_net' ),
			array( $this, 'desk_net_category_matching_html' ),
			'wp_desk_net_category_matching'
		);

		add_settings_field(
			'wp_desk_net_desk_net_to_wp_category',
			__( 'Desk-Net to WordPress', 'desk_net' ),
			array( $this, 'desk_net_to_wp_category_html' ),
			'wp_desk_net_category_matching',
			'wp_desk_net_category_list'
		);

		add_settings_field(
			'wp_desk_net_wp_to_desk_net_category',
			__( 'WordPress to Desk-Net', 'desk_net' ),
			array( $this, 'wp_to_desk_net_category_html' ),
			'wp_desk_net_category_matching',
			'wp_desk_net_category_list'
		);

		// The "Content Settings" page.
		register_setting( 'wp_desk_net_content_matching',
			'wp_desk_net_content_list', array(
				$this,
				'desk_net_save_matching_settings',
			) );

		add_settings_section(
			'wp_desk_net_content_list',
			__( 'Content import settings', 'desk_net' ),
			array( $this, 'desk_net_content_matching_html' ),
			'wp_desk_net_content_matching'
		);

		add_settings_field(
			'wp_desk_net_content_settings_list',
			__( 'Export setting per content type:', 'desk_net' ),
			array( $this, 'desk_net_content_setting_html' ),
			'wp_desk_net_content_matching',
			'wp_desk_net_content_list'
		);

		// The "Slug Syncing" page.
		register_setting(
			'desk_net_slug_option', // Option group.
			'wp_desk_net_slug_list', // Option name.
			array( $this, 'desk_net_save_matching_settings' ) // Sanitize.
		);

		register_setting(
			'desk_net_slug_option', // Option group.
			'wp_desk_net_initial_slug_syncing', // Option name.
			array( $this, 'save_initial_slug_syncing' ) // Sanitize.
		);

		add_settings_section(
			'desk_net_slug_section', // ID.
			__( 'Slug Syncing', 'desk_net' ), // Title.
			array( $this, 'desk_net_slug_info' ), // Callback.
			'desk_net_slug_syncing' // Page.
		);

		add_settings_field(
			'desk_net_slug_field', // ID
			'', // Title.
			array( $this, 'desk_net_slug_callback' ), // Callback.
			'desk_net_slug_syncing', // Page.
			'desk_net_slug_section' // Section.
		);

		add_settings_field(
			'desk_net_initial_slug_syncing', // ID
			'', // Title.
			array( $this, 'desk_net_initial_slug_checkbox' ), // Callback.
			'desk_net_slug_syncing', // Page.
			'desk_net_slug_section' // Section.
		);

		// The "Desk-Net ID in URL" page.
		register_setting( 'wp_desk_net_permalinks',
			'wp_desk_net_id_in_permalink_option', array(
				$this,
				'save_permalink',
			) );

		add_settings_section(
			'wp_desk_net_permalink',
			__( 'Desk-Net ID in URL', 'desk_net' ),
			array( $this, 'wp_desk_net_permalink_section_html' ),
			'wp_desk_net_permalinks'
		);

		add_settings_field(
			'wp_desk_net_id_in_permalink',
			__( 'Desk-Net ID in URL', 'desk_net' ),
			array( $this, 'wp_desk_net_id_in_permalink_html' ),
			'wp_desk_net_permalinks',
			'wp_desk_net_permalink',
			array(
				'Insert Desk-Net ID in URL',
			)
		);
	}

	/**
	 * Perform generate API credentials.
	 */
	function generate_api_credentials() {
		update_option( 'wp_desk_net_api_key', uniqid( 'API_', true ) );
		update_option( 'wp_desk_net_api_secret', uniqid( '', true ) );

		add_settings_error(
			'wp_desk_net__settings',
			esc_attr( 'credentials_generated' ),
			__( 'New credentials successfully generated.' ),
			'updated'
		);
	}

	/**
	 * Perform save authorization form.
	 *
	 * @param array $args
	 *   The fields form.
	 */
	function desk_net_save_authorization( $args ) {
		if ( null !== $args ) {
			update_option( 'wp_desk_net_user_login',
				$args['wp_desk_net_user_login'] );
			update_option( 'wp_desk_net_user_password',
				$args['wp_desk_net_user_password'] );
			delete_option( 'wp_desk_net_token' );
			$token
				= $this->desk_net_check_validate_token( $args['wp_desk_net_user_login'],
				$args['wp_desk_net_user_password'] );

			if ( false != $token ) {
				$message = __( 'Connection successfully established.',
					'desk_net' );
				$type    = 'updated';
			} else {
				$message = __( 'Connection could not be established.',
					'desk_net' );
				$type    = 'error';
			}
			add_settings_error(
				'wp_desk_net__authorization',
				esc_attr( 'authorization_updated' ),
				$message,
				$type
			);
		}
	}

	/**
	 * Perform save matching settings.
	 *
	 * @param array $args
	 *   The fields form.
	 */
	function desk_net_save_matching_settings( $args ) {
		if ( null !== $args ) {
			foreach ( $args as $key => $value ) {
				update_option( $key, $value );
			}
			add_settings_error(
				'desk_net_save_matching_settings',
				esc_attr( 'matching_updated' ),
				__( 'Settings saved.' ),
				'updated'
			);
		}
	}

	/**
	 * Perform content on page.
	 */
	function desk_net_settings_section_html() {
		esc_html_e( 'Use these credentials in Desk-Net on the Advanced Settings tab of the',
			'desk_net' );
		echo ' <a href="https://www.desk-net.com/objectsPage.htm" target="_blank">platform</a> ';
		esc_html_e( 'you are connecting to this WordPress website.',
			'desk_net' );
	}

	/**
	 * Print the Slug text info.
	 */
	function desk_net_slug_info() {
		esc_html_e( 'Desk-Net syncs the Title, SEO Title and the URL Alias with the',
			'desk_net' );
		echo ' <a href="https://support.desk-net.com/hc/en-us/articles/115003545092" target="_blank">slug feature</a> ';
		esc_html_e( 'in Desk-Net. Please define what field should be primarily synced to Desk-Net in case of conflicts.',
			'desk_net' );
	}

	/**
	 * Perform content on page.
	 */
	function desk_net_status_matching_html() {
		esc_html_e( 'Use this page to match publication statuses in Desk-Net to those in WordPress and vice versa.',
			'desk_net' );
	}

	/**
	 * Perform content on page.
	 */
	function desk_net_content_matching_html() {
		esc_html_e( 'If your Desk-Net users upload content such as pictures, video files, text etc. 
					to Desk-Net such content can be imported into WordPress.',
			'desk_net' );
		echo '</br></br>';
		esc_html_e( 'Here you can define if file based content such as picture or video files should 
          be imported straight into a post’s body or if they should be made available in the 
          Media section of your WordPress site only.', 'desk_net' );
		echo '</br></br>';
		esc_html_e( 'There will be no content import once a story has been published or if it has been 
          scheduled for automatic publication at a pre-defined date/time.',
			'desk_net' );
	}

	/**
	 * Perform content on page.
	 */
	function desk_net_category_matching_html() {
		esc_html_e( 'Use this page to match categories in Desk-Net to those in WordPress and vice versa.',
			'desk_net' );
	}

	/**
	 * Perform template for fields Desk-Net to WordPress status.
	 */
	function desk_net_to_wp_status_html() {
		$desk_net_to_wp = [];
		$wp_to_desk_net = [];

		$desk_net_status_list
			           = get_option( 'wp_desk_net_desk-net-list-active-status' );
		$post_statuses = get_post_statuses();

		foreach ( $desk_net_status_list as $key => $value ) {
			$desk_net_to_wp[ $value['id'] ]['name']
				= $value['name'];
		}

		$desk_net_to_wp['removed']['name'] = __( 'Deleted/removed',
			'desk_net' );

		foreach ( $post_statuses as $key => $value ) {
			$wp_to_desk_net[ $key ]['name'] = $value;
		}

		$this->desk_net_remove_item_from_status_list( $wp_to_desk_net,
			'private' );

		if ( empty( get_option( 'wp_desk_net_status_desk_net_to_wp_5' ) ) ) {
			update_option( 'wp_desk_net_status_desk_net_to_wp_5', 'publish' );
		}

		if ( ! empty( $wp_to_desk_net ) && ! empty( $desk_net_to_wp ) ) {
			desk_net_template_matching_pages( $desk_net_to_wp, $wp_to_desk_net,
				'status', '_desk_net_to_wp_', 'draft' );
		} else {
			esc_html_e( "Sorry, we couldn't load statuses", 'desk_net' );
		}
	}

	/**
	 * Perform template for fields WordPress to Desk-Net status.
	 */
	function wp_to_desk_net_status_html() {
		$desk_net_to_wp = [];
		$wp_to_desk_net = [];

		$desk_net_active_status_list
			           = get_option( 'wp_desk_net_desk-net-list-active-status' );
		$desk_net_deactivated_status_list
			           = get_option( 'wp_desk_net_desk-net-list-deactivated-status' );
		$post_statuses = get_post_statuses();

		if ( ! empty( $desk_net_active_status_list ) ) {
			foreach ( $desk_net_active_status_list as $value ) {
				$desk_net_to_wp[ $value['id'] ]['name']
					= $value['name'];
			}
		}
		if ( ! empty( $desk_net_deactivated_status_list ) ) {
			foreach ( $desk_net_deactivated_status_list as $value ) {
				$desk_net_to_wp[ $value['id'] ]['name']
					= $value['name'];
			}
		}
		foreach ( $post_statuses as $key => $value ) {
			$wp_to_desk_net[ $key ]['name'] = $value;
		}

		$wp_to_desk_net
			= $this->desk_net_add_scheduled_status( $wp_to_desk_net );

		$this->desk_net_remove_item_from_status_list( $wp_to_desk_net,
			'private' );
		// Removing old matching for Private status from previous plugin version.
		delete_option( 'wp_desk_net_status_wp_to_desk_net_private' );

		if ( empty( get_option( 'wp_desk_net_status_wp_to_desk_net_publish' ) ) ) {
			update_option( 'wp_desk_net_status_wp_to_desk_net_publish', 5 );
		}

		if ( ! empty( $wp_to_desk_net ) && ! empty( $desk_net_to_wp ) ) {
			desk_net_template_matching_pages( $wp_to_desk_net, $desk_net_to_wp,
				'status', '_wp_to_desk_net_', '1' );
		} else {
			esc_html_e( "Sorry, we couldn't load statuses", 'desk_net' );
		}
	}

	/**
	 * Adding the Scheduled status to the WP status list.
	 *
	 * @param array $wp_to_desk_net
	 *     The WP status list.
	 *
	 * @return array
	 */
	function desk_net_add_scheduled_status( $wp_to_desk_net ) {
		$position = array_search( $wp_to_desk_net['publish']['name'],
			$wp_to_desk_net );

		return array_slice( $wp_to_desk_net, 0, $position - 1 )
		       + [ 'scheduled' => [ 'name' => 'Scheduled' ] ]
		       + array_slice( $wp_to_desk_net, $position - 1 );
	}

	/**
	 *
	 * Removing status by name from the WP status list.
	 *
	 * @param array  $wp_to_desk_net
	 *     The WP status list.
	 * @param string $status_name
	 *     The WP status name.
	 */
	function desk_net_remove_item_from_status_list(
		&$wp_to_desk_net, $status_name
	) {
		unset( $wp_to_desk_net[ $status_name ] );
	}

	/**
	 * Perform template for fields Desk-Net to WordPress content.
	 */
	function desk_net_content_setting_html() {
		$element_menu_list = [];
		$menu_values_list  = [];

		$element_menu_list['picture-files']['name'] = __( 'Picture files',
			'desk_net' );
		$element_menu_list['video-files']['name']   = __( 'Video files',
			'desk_net' );
		$element_menu_list['other-files']['name']   = __( 'Other files',
			'desk_net' );

		$menu_values_list['body-media']['name']
			= __( 'Import into post body and Media area', 'desk_net' );
		$menu_values_list['only-media']['name']
			= __( 'Store in Media area only', 'desk_net' );

		desk_net_template_matching_pages( $element_menu_list, $menu_values_list,
			'content', '_', 'body-media' );
	}

	/**
	 * Perform template for fields Desk-Net to WordPress category.
	 */
	function desk_net_to_wp_category_html() {
		$wp_to_desk_net = [];
		$desk_net_to_wp = [];

		$wp_category_list = get_terms( 'category',
			'orderby=namet&order=ASC&hide_empty=0' );
		$wp_category_list = json_decode( wp_json_encode( $wp_category_list,
			JSON_UNESCAPED_UNICODE ), true );

		$desk_net_category_list
			= get_option( 'wp_desk_net_desk_net_category_list' );

		$wp_to_desk_net['no_category']['name']   = 'No category';
		$wp_to_desk_net['do_not_import']['name'] = 'Do not import';
		$desk_net_to_wp['no_category']['name']   = 'No category';

		foreach ( $wp_category_list as $value ) {
			$wp_to_desk_net[ $value['term_id'] ]['name']
				= $value['name'];
			if ( 0 !== (int) $value['parent'] ) {
				$wp_to_desk_net[ $value['term_id'] ]['parent']
					= $value['parent'];
			}
		}
		foreach ( $desk_net_category_list as $value ) {
			$desk_net_to_wp[ $value['id'] ]['name']
				= $value['name'];
			if ( isset( $value['category'] ) ) {
				$desk_net_to_wp[ $value['id'] ]['parent']
					= $value['category'];
			}
		}
		if ( ! empty( $wp_to_desk_net ) && ! empty( $desk_net_to_wp ) ) {
			desk_net_template_matching_pages( $desk_net_to_wp, $wp_to_desk_net,
				'category', '_desk_net_to_wp_', 'no_category' );
		} else {
			esc_html_e( "Sorry, we can't load categories", 'desk_net' );
		}
	}

	/**
	 * Get the settings option array and print one of its values.
	 */
	public function desk_net_slug_callback() {
		$desk_net_slug       = [];
		$desk_net_slug_value = [];

		// Slug field Desk-Net.
		$desk_net_slug['slug_syncing']['name'] = __( 'Slug (Desk-Net)',
			'desk_net' );

		// Set values.
		$desk_net_slug_value['url_alias']['name'] = __( 'URL Alias',
			'desk_net' );
		$desk_net_slug_value['title']['name']     = __( 'Title', 'desk_net' );

		// Printing template.
		desk_net_template_matching_pages( $desk_net_slug, $desk_net_slug_value,
			'slug', '_', 'url_alias' );
	}

	/**
	 * Perform template for fields WordPress to Desk-Net category.
	 */
	function wp_to_desk_net_category_html() {
		$desk_net_to_wp = [];
		$wp_to_desk_net = [];

		$wp_category_list = get_terms( 'category',
			'orderby=namet&order=ASC&hide_empty=0' );
		$wp_category_list = json_decode( wp_json_encode( $wp_category_list,
			JSON_UNESCAPED_UNICODE ), true );

		$desk_net_category_list
			= get_option( 'wp_desk_net_desk_net_category_list' );

		$desk_net_to_wp['No category']['name']   = 'No category';
		$desk_net_to_wp['Do not import']['name'] = 'Do not import';

		foreach ( $wp_category_list as $value ) {
			$wp_to_desk_net[ $value['term_id'] ]['name']
				= $value['name'];
			if ( 0 !== (int) $value['parent'] ) {
				$wp_to_desk_net[ $value['term_id'] ]['parent']
					= $value['parent'];
			}
		}

		foreach ( $desk_net_category_list as $value ) {
			$desk_net_to_wp[ $value['id'] ]['name']
				= $value['name'];
			if ( isset( $value['category'] ) ) {
				$desk_net_to_wp[ $value['id'] ]['parent']
					= $value['category'];
			}
		}

		if ( ! empty( $wp_to_desk_net ) && ! empty( $desk_net_to_wp ) ) {
			desk_net_template_matching_pages( $wp_to_desk_net, $desk_net_to_wp,
				'category', '_wp_to_desk_net_', 'No category' );
		} else {
			esc_html_e( "Sorry, we can't load categories", 'desk_net' );
		}
	}

	/**
	 * Perform content on page.
	 */
	function authorization_section_html() {
		esc_html_e( 'Enter here the API credentials of your Desk-Net account (not your personal 
			Desk-Net user credentials). If you do not have these API credentials please request them from',
			'desk_net' );
		echo ' <a href="mailto:support@desk-net.com">support@desk-net.com</a>';
	}

	/**
	 * Perform content on page.
	 */
	function wp_desk_net_login_fidel_html() {
		echo '<input type="text" id="wp_desk_net_user_login" 
			name="wp_desk_net__authorization[wp_desk_net_user_login]" value="'
		     . esc_attr( get_option( 'wp_desk_net_user_login', '' ) ) . '" />';
	}

	/**
	 * Perform content on page.
	 */
	function wp_desk_net_password_fidel_html() {
		echo '<input type="password" id="wp_desk_net_user_password" 
			name="wp_desk_net__authorization[wp_desk_net_user_password]" value="'
		     . esc_attr( get_option( 'wp_desk_net_user_password', '' ) )
		     . '" autocomplete="new-password" />';
	}

	/**
	 * Perform content on page.
	 */
	function wp_desk_net_permalink_section_html() {
		esc_html_e( 'You can have this plugin automatically insert the Desk-Net ID into the URL of new posts.',
			'desk_net' );
	}

	/**
	 * Perform template for fields WordPress permalink page.
	 */
	function wp_desk_net_id_in_permalink_html( $args ) {
		echo '<input type="checkbox" id="wp_desk_net_id_in_permalink" 
			name="wp_desk_net_id_in_permalink_option[wp_desk_net_id_in_permalink]" '
		     . checked( 1, get_option( 'wp_desk_net_id_in_permalink', false ),
				false ) . '/>';
		echo '<label for="wp_desk_net_id_in_permalink"> ' . esc_html( $args[0] )
		     . '</label>';
		$this->status_upload = false;
	}

	/**
	 * Perform save the 'Initial slug syncing' option field.
	 *
	 * @param array $args
	 *   The fields form.
	 */
	public function save_initial_slug_syncing( $args ) {
		if ( null !== $args ) {
			update_option( 'wp_desk_net_initial_slug_syncing_checkbox', 1 );
		} else {
			update_option( 'wp_desk_net_initial_slug_syncing_checkbox', 0 );
		}
	}

	/**
	 * Perform save permalink settings.
	 *
	 * @param array $args
	 *   The fields form.
	 */
	function save_permalink( $args ) {
		if ( ! $this->status_upload ) {
			if ( null !== $args ) {
				update_option( 'wp_desk_net_id_in_permalink', 1 );
			} else {
				update_option( 'wp_desk_net_id_in_permalink', 0 );
			}
			$this->status_upload = true;
		}
	}

	/**
	 * Perform content on page.
	 */
	function api_url_html() {
		echo '<div class="dn-d-flex dn-clipboard">
                <div class="dn-wp-credentials">' . esc_url( site_url()
		                                                    . '/wp-json/wp-desk-net/v1' )
		     . '</div>
                <div class="dn-button dn-ml-15 dn-relative c-dn-copy-text" title="Copy API URL">
                    <p class="c-dn-copied">Copied</p><i class="dn-clipboard-icon"></i>
                </div>
              </div>';
	}

	/**
	 * Perform content on page.
	 */
	function api_key_input_html() {
		echo '<div class="dn-d-flex dn-clipboard">
                <div class="dn-wp-credentials">'
		     . esc_html( get_option( 'wp_desk_net_api_key' ) ) . '</div>
                <div class="dn-button dn-ml-15 dn-relative c-dn-copy-text" title="Copy API Key">
                    <p class="c-dn-copied">Copied</p><i class="dn-clipboard-icon"></i>
                </div>
              </div>';
	}

	/**
	 * Perform content on page.
	 */
	function api_secret_input_html() {
		echo '<div class="dn-d-flex dn-clipboard">
                <div class="dn-wp-credentials">'
		     . esc_html( get_option( 'wp_desk_net_api_secret' ) ) . '</div>
                <div class="dn-button dn-ml-15 dn-relative c-dn-copy-text" title="Copy API Secret">
                    <p class="c-dn-copied">Copied</p><i class="dn-clipboard-icon"></i>
                </div>
              </div>';
	}

	/**
	 * Perform default value after init WP-Desk-Net plugin.
	 */
	function wp_desk_net_activate() {
		add_option( 'wp_desk_net_api_key', uniqid( 'API_', true ) );
		add_option( 'wp_desk_net_api_secret', uniqid( '', true ) );
	}

	/**
	 * Perform Settings link on plugin page.
	 *
	 * @param string $links
	 *   The plugin action links.
	 *
	 * @return string
	 */
	function add_action_links( $links ) {
		$plugin_links = array(
			'<a href="' . admin_url( 'admin.php?page=wp-desk-net' ) . '">'
			. __( 'Settings', 'desk_net' ) . '</a>',
		);

		return array_merge( $links, $plugin_links );
	}

	/**
	 * Perform additional text fields for post in WordPress.
	 *
	 * @param object $post
	 */
	function desk_net_description( $post ) {
		// Updating special symbols on HTML essence.
		$data = $this->get_desk_net_post_data( $post );

		if ( $data
		     && is_plugin_active( 'classic-editor/classic-editor.php' )
		) {
			?>
            <div class="desk-net-info-block">
                <strong class="desk-net-desc">
                    <a href="<?php echo esc_url( Desk_Net_Plugin_Settings::DN_BASE_URL
					                             . '/mySchedulePage.htm?fragment=de'
					                             . $data['desk_net_id'] ); ?>"
                       target="_blank"><img
                                src="<?php echo esc_html( plugin_dir_url( __FILE__ ) ); ?>../img/favicon.ico"
                                alt="Desk-Net"/>
                    </a>
					<?php
					if ( get_post_meta( $post->ID, 'wp_desk_net_remove_status',
							true ) === 'removed'
					) {
						echo esc_html__( 'Deleted/removed – ', 'desk_net' );
						echo $data['desk_net_description'];
					} else {
						echo $data['desk_net_description'];
					}
					?>
                </strong>
                <div class="desk-net-id"><?php echo esc_html__( 'Status in Desk-Net: ',
						'desk_net' ); ?>
                    <strong><?php echo esc_html( $data['desk_net_status_name'] ); ?></strong>
                    <a href="<?php echo esc_url( Desk_Net_Plugin_Settings::DN_BASE_URL
					                             . '/mySchedulePage.htm?fragment=de'
					                             . $data['desk_net_id'] ); ?>"
                       target="_blank"><?php echo esc_html__( 'View in Desk-Net ',
							'desk_net' ); ?></a>
                </div>
            </div>
			<?php
		}
	}

	/**
	 * Perform delete filter and add query message param.
	 *
	 * @param string $location
	 *
	 * @return string
	 */
	public function add_notice_query_var( $location ) {
		remove_filter( 'redirect_post_location', array(
			$this,
			'add_notice_query_var',
		), 99 );

		return add_query_arg( array(
			'message_id' => $this->error_message,
			'warning_id' => implode( '_', $this->warning_message ),
		), $location );
	}

	/**
	 * Performing check Post on 'insert Slug' action.
	 *
	 * @param int $post_id Post ID.
	 *
	 * @return boolean
	 */
	public function is_slug_syncing( $post_id ) {
		$story_id = get_post_meta( $post_id, 'story_id', true );
		if ( $story_id && $this->is_initial_slug_syncing() ) {
			return false;
		}

		return true;
	}

	/**
	 * Checking the enables status to the 'initial slug syncing' option.
	 *
	 * @return boolean
	 */
	private function is_initial_slug_syncing() {
		$value = get_option( 'wp_desk_net_initial_slug_syncing_checkbox' );

		return (bool) $value;
	}

	/**
	 * Printing Initial slug checkbox.
	 */
	private function desk_net_initial_slug_checkbox() {
		$checkbox = new Desk_Net_HTML();
		$value
		          = ( get_option( 'wp_desk_net_initial_slug_syncing_checkbox' ) )
			? 'checked' : '';
		$checkbox->desk_net_checkbox(
			'initial_slug_syncing',
			'initial_slug_syncing_checkbox',
			'initial_slug_syncing',
			$value,
			__( 'Only allow initial slug syncing', 'desk_net' ),
			'left',
			[ 'dn-default-font-style', 'dn-pr-33', 'dn-font-weight-bold' ] );
	}

	/**
	 * Getting the date from Desk-Net story.
	 *
	 * @param array $date_arr
	 *     The story date.
	 *
	 * @return boolean|string
	 */
	public function get_story_date( $date_arr ) {
		if ( empty( $date_arr['date'] ) ) {
			return false;
		}

		$format = 'Y-m-d H:i';

		if ( isset( $date_arr['time'] ) && ! empty( $date_arr['time'] ) ) {
			$date = $date_arr['date'] . ' ' . $date_arr['time'];
		} else {
			$default_date = strtotime( $date_arr['date'] . ' ' . '23:59' );
			$default_date = date( $format, $default_date );
			$date         = strtotime( $default_date )
			                - get_option( 'gmt_offset' ) * 60
			                  * 60; // The default Time create post if time on Desk-Net is empty.
			$date         = date( $format, $date );
		}

		$check_date = DateTime::createFromFormat( $format, $date );

		if ( array_sum( $check_date->getLastErrors() ) ) {
			return false;
		}

		return $check_date->format( 'Y-m-d H:i:s' );
	}

	/**
	 * Setting a new date to the Post attribute.
	 *
	 * @param array  $post_data
	 *     The Post attribute.
	 * @param string $date
	 *     The new date.
	 *
	 * @return array
	 */
	public function set_post_date( $post_data, $date ) {
		$post_data['post_date_gmt'] = $date;

		$add_timezone = strtotime( $post_data['post_date_gmt'] )
		                + get_option( 'gmt_offset' ) * 60 * 60;

		$post_data['post_date'] = date( 'Y-m-d H:i:s',
			$add_timezone );

		return $post_data;
	}
}

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly.
