<?php

/**
 * @package Desk_Net_Delete_Methods.
 * @author  Desk-Net GmbH.
 */

include_once plugin_dir_path( __FILE__ ) . 'class-desk-net-core.php';

class Desk_Net_Delete_Methods extends Desk_Net_Core {

	/**
	 * Perform shape list of deleted items.
	 *
	 * @param array  $new_items_list
	 *   The new elements list.
	 * @param array  $save_items_list
	 *   The old elements list.
	 * @param array  $wp_items_list
	 *   The WP elements list.
	 *   string array[key]
	 *   The key has id or Slug element value.
	 * @param string $type
	 */
	public function desk_net_shape_deleted_items(
		$new_items_list, $save_items_list, $wp_items_list, $type
	) {
		$desk_net_delete_element_list = array();

		if ( ! empty( $save_items_list ) ) {
			foreach ( $save_items_list as $value ) {
				$in_stock = false;
				foreach ( $new_items_list as $content ) {
					if ( $value['id'] === $content['id'] ) {
						$in_stock = true;
					}
				}
				if ( ! $in_stock ) {
					$desk_net_delete_element_list[] = $value['id'];
				}
			}
		}
		if ( ! empty( $desk_net_delete_element_list ) ) {
			$desk_net_delete_element_list
				= $this->check_sub_items( $desk_net_delete_element_list );
			$this->desk_net_delete_items( $desk_net_delete_element_list,
				$wp_items_list, $type );
		}
	}

	/**
	 * Perform checking parent elements in the list of elements to be deleted.
	 *
	 * @param array $desk_net_delete_element_list
	 *   The list delete elements.
	 *
	 * @return array $desk_net_delete_element_list
	 */
	public function check_sub_items( $desk_net_delete_element_list ) {
		$save_items_list_for_platform
			                = get_option( 'wp_desk_net_desk_net_category_list' );
		$under_sub_category = array();

		if ( ! empty( $save_items_list_for_platform ) ) {
			foreach ( $save_items_list_for_platform as $key => $content ) {
				foreach ( $desk_net_delete_element_list as $value ) {
					if ( isset( $content['category'] )
					     && $value === $content['category']
					) {
						$desk_net_delete_element_list[] = $content['id'];
						$under_sub_category[]           = $content['id'];
					}
				}
			}
		}
		if ( ! empty( $under_sub_category ) ) {
			$this->check_sub_items( $under_sub_category );
		}
		
		return $desk_net_delete_element_list;
	}

	/**
	 * Perform deleted items and mapping category.
	 *
	 * @param array  $desk_net_delete_element_list
	 *   The list delete elements.
	 * @param array  $wp_items_list
	 *   The WP elements list.
	 * @param string $type
	 *   The elements type in list.
	 *
	 */
	public function desk_net_delete_items(
		$desk_net_delete_element_list, $wp_items_list, $type
	) {
		if ( ! empty( $wp_items_list )
		     && ! empty( $desk_net_delete_element_list )
		) {
			foreach ( $desk_net_delete_element_list as $content ) {
				foreach ( $wp_items_list as $value ) {
					if ( get_option( 'wp_desk_net_' . $type . '_wp_to_desk_net_'
					                 . $value ) === $content
					) {
						update_option( 'wp_desk_net_' . $type
						               . '_wp_to_desk_net_' . $value,
							'No ' . $type );
					}
				}
				delete_option( 'wp_desk_net_' . $type . '_desk_net_to_wp_'
				               . $content );
			}
		}
	}
}
