"use strict";

let el = wp.element.createElement;
let __ = wp.i18n.__;
let registerPlugin = wp.plugins.registerPlugin;
let getPlugin = wp.plugins.getPlugin;
let PluginDocumentSettingPanel = wp.editPost.PluginDocumentSettingPanel;
let customMeta = wp.data.select('core/editor').getEditedPostAttribute('meta');
let wasSavingPost = wp.data.select('core/editor').isSavingPost();
let wasAutosavingPost = wp.data.select('core/editor').isAutosavingPost();

const messageList = {
    1: __('Story update successfully sent to Desk-Net'),
    2: __('Cannot update story in Desk-Net. The WordPress plugin could not find a corresponding story ID in Desk-Net. ' +
        'Code: 01'),
    3: __('Cannot update story in Desk-Net. There is no corresponding story in Desk-Net. Code: 02'),
    4: __('Cannot update the story in Desk-Net. Reason unknown. Please contact Desk-Net support at ' +
        'support@desk-net.com. Code: 03'),
    5: __('The Desk-Net API login credentials are not valid or have not been entered. Please check ' +
        'the settings on the page Desk-Net Credentials in the Desk-Net plugin. Code: 04'),
    6: __('Cannot create story in Desk-Net. Reason unknown. Code: 05'),
    7: __('Cannot create story in Desk-Net. Publication date doesn\'t match platform schedule.'),
    8: __('Cannot update story in Desk-Net. Publication date doesn\'t match platform schedule.')
};
// determine whether to show notice.
const savingPost = wp.data.subscribe(function () {
    const isSavingPost = wp.data.select('core/editor').isSavingPost();
    const isAutosavingPost = wp.data.select('core/editor').isAutosavingPost();
    // Save metaboxes on save completion, except for autosaves that are not a post preview.
    const shouldTriggerTemplateNotice = (wasSavingPost && !isSavingPost && !wasAutosavingPost);
    // Save current state for next inspection.
    customMeta = wp.data.select('core/editor').getEditedPostAttribute('meta');
    wasSavingPost = isSavingPost;
    wasAutosavingPost = isAutosavingPost;

    if (shouldTriggerTemplateNotice) {
        const notice_id = JSON.parse(customMeta.desk_net_notice_message_id);
        const storyId = JSON.parse(customMeta.story_id);

        if ('' !== storyId && null !== storyId && !getPlugin('desk-net-document-setting-plugin')) {
            registerPlugin('desk-net-document-setting-plugin', {
                render: DeskNetDocumentSettingPlugin
            });
        }

        if (JSON.parse(customMeta.wp_desk_net_remove_status) !== 'removed' && notice_id !== '0') {
            wp.data.dispatch('core/notices').createNotice(
                ('1' === notice_id) ? 'success' : 'error', // Can be one of: success, info, warning, error.
                messageList[notice_id], // Text string to display.
                {
                    id: 'desk_net_template_update', //assigning an ID prevents the notice from being added repeatedly
                    isDismissible: true, // Whether the user can dismiss the notice.
                },
            );
        }
    } else if (!getPlugin('desk-net-document-setting-plugin')
        && customMeta !== undefined
        && customMeta.story_id !== undefined
        && '' !== JSON.parse(customMeta.story_id)
        && null !== JSON.parse(customMeta.story_id)) {
        registerPlugin('desk-net-document-setting-plugin', {
            render: DeskNetDocumentSettingPlugin
        });
    }
});

const DeskNetDocumentSettingPlugin = () => {
    let deskNetDescription = __(JSON.parse(customMeta.desk_net_description));

    if (JSON.parse(customMeta.wp_desk_net_remove_status) === 'removed') {
        deskNetDescription = __('Deleted/removed – ') + deskNetDescription;
    }

    return el(
        PluginDocumentSettingPanel,
        {
            className: 'desk-net-document-setting-plugin',
            title: 'Desk-Net',
            icon: ' '
        },
        el(
            'div',
            {class: 'desk-net-gutenberg-title'},
            el('a', {href: desk_net.view_in_desk_net, target: '_blank'},
                el('img', {
                    src: desk_net.icon_src,
                    alt: 'Desk-Net'
                }),
            ),
            el('strong', null, deskNetDescription)
        ),
        el(
            'div',
            {class: 'desk-net-gutenberg-panel'},
            el('span', null, __('Status in Desk-Net:')),
            el('strong', null, __(JSON.parse(customMeta.desk_net_status_name))),
        ),
        el(
            'a',
            {href: desk_net.view_in_desk_net + JSON.parse(customMeta.story_id), target: '_blank'},
            __('View in Desk-Net'),
        ),
    );
}
